@@ -28,7 +28,7 @@
 #include <gsf/gsf-impl-utils.h>
 
 #include <glib-object.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gmodule.h>
 
 #include <stdlib.h>
@@ -637,6 +637,7 @@ go_plugin_read_service_list (GOPlugin *plugin, xmlNode *tree, ErrorInfo **ret_er
 
 	g_return_val_if_fail (tree != NULL, NULL);
 
+	GO_INIT_RET_ERROR_INFO (ret_error);
 	node = e_xml_get_child_by_name (tree, (xmlChar *)"services");
 	if (node == NULL)
 		return NULL;
@@ -1038,6 +1039,8 @@ go_plugin_deactivate (GOPlugin *plugin, ErrorInfo **ret_error)
 			go_plugin_use_unref (plugin_dependency_get_plugin (dep));
 		);
 		if (plugin->loader != NULL) {
+			if (go_plugin_loader_is_base_loaded (plugin->loader))
+				go_plugin_loader_unload_base (plugin->loader, ret_error);
 			g_object_unref (plugin->loader);
 			plugin->loader = NULL;
 		}
@@ -1522,8 +1525,9 @@ go_plugins_get_plugin_by_id (const gchar *plugin_id)
 }
 
 /**
- * plugin_db_mark_plugin_for_deactivation:
- * ...
+ * plugin_db_mark_plugin_for_deactivation :
+ * @plugin:
+ * @mark:
  */
 void
 go_plugin_db_mark_plugin_for_deactivation (GOPlugin *plugin, gboolean mark)
@@ -1543,8 +1547,8 @@ go_plugin_db_mark_plugin_for_deactivation (GOPlugin *plugin, gboolean mark)
 }
 
 /**
- * plugin_db_is_plugin_marked_for_deactivation:
- * ...
+ * plugin_db_is_plugin_marked_for_deactivation :
+ * @plugin: 
  */
 gboolean
 go_plugin_db_is_plugin_marked_for_deactivation (GOPlugin *plugin)
