@@ -40,6 +40,7 @@
 #include <goffice/utils/go-font.h>
 #include <goffice/utils/go-math.h>
 #include <goffice/app/go-plugin-service.h>
+#include <goffice/component/go-component-factory.h>
 #include <gsf/gsf-utils.h>
 
 #include "goffice-paths.h"
@@ -54,19 +55,19 @@ static char const *libgoffice_locale_dir = GOFFICE_LOCALEDIR;
 static char const *libgoffice_lib_dir    = GOFFICE_LIBDIR;
 
 gchar const *
-go_sys_data_dir ()
+go_sys_data_dir (void)
 {
 	return libgoffice_data_dir;
 }
 
 gchar const *
-go_sys_icon_dir ()
+go_sys_icon_dir (void)
 {
 	return libgoffice_icon_dir;
 }
 
 gchar const *
-go_sys_lib_dir ()
+go_sys_lib_dir (void)
 {
 	return libgoffice_lib_dir;
 }
@@ -80,8 +81,8 @@ libgoffice_init (void)
 		return;
 
 #ifdef G_OS_WIN32
-{
-	gchar *dir = g_win32_get_package_installation_directory (NULL, "libgoffice-1-0.dll");
+	{
+	gchar *dir = g_win32_get_package_installation_directory (NULL, "libgoffice-0-3.dll");
 	libgoffice_data_dir = g_build_filename (dir,
 		"share", "goffice", GOFFICE_VERSION, NULL);
 	libgoffice_icon_dir = g_build_filename (dir,
@@ -91,10 +92,11 @@ libgoffice_init (void)
 	libgoffice_lib_dir = g_build_filename (dir,
 		"lib", "goffice", GOFFICE_VERSION, NULL);
 	g_free (dir);
-}
+	}
 #endif
 
 	bindtextdomain (GETTEXT_PACKAGE, libgoffice_locale_dir);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	go_fonts_init ();
 	go_math_init ();
 	gsf_init ();
@@ -102,6 +104,7 @@ libgoffice_init (void)
 	/* keep trigger happy linkers from leaving things out */
 	plugin_services_init ();
 	gog_plugin_services_init ();
+	goc_plugin_services_init ();
 	(void) GOG_GRAPH_TYPE;
 	(void) GOG_CHART_TYPE;
 	(void) GOG_PLOT_TYPE;
@@ -119,7 +122,8 @@ libgoffice_init (void)
 	(void) GO_DATA_SCALAR_VAL_TYPE;
 	(void) GO_DATA_SCALAR_STR_TYPE;
 	gog_themes_init	();
-	number_format_init ();
+	go_number_format_init ();
+	go_currency_date_format_init ();
 }
 
 void
@@ -127,8 +131,10 @@ libgoffice_shutdown (void)
 {
 	gog_themes_shutdown ();
 	go_fonts_shutdown ();
+	goc_plugin_services_shutdown ();
 	gog_plugin_services_shutdown ();
-	number_format_shutdown ();
+	go_currency_date_format_shutdown ();
+	go_number_format_shutdown ();
 #ifdef G_OS_WIN32
 	g_free (libgoffice_data_dir);
 	g_free (libgoffice_icon_dir);
