@@ -30,6 +30,11 @@
 
 G_BEGIN_DECLS
 
+typedef struct  {
+		gboolean tick_in, tick_out;
+		int size_pts;
+} GogAxisTickProperties;
+
 typedef struct {
 	GogStyledObject	 base;
 
@@ -40,10 +45,7 @@ typedef struct {
 	unsigned 	   crossed_axis_id;
 	GogDatasetElement  cross_location;
 	
-	struct {
-		gboolean tick_in, tick_out;
-		int size_pts;
-	} major, minor;
+	GogAxisTickProperties major, minor;
 	gboolean major_tick_labeled;
 } GogAxisBase;
 
@@ -58,9 +60,13 @@ GType gog_axis_base_get_type (void);
 GogAxisPosition 	gog_axis_base_get_position (GogAxisBase *axis_base);
 void		    	gog_axis_base_set_position (GogAxisBase *axis_base, GogAxisPosition position);
 
-void			gog_axis_base_set_label_angle (GogAxisBase *axis_base, double angle);
+typedef struct {
+	GogView		base;
+
+	double		x_start, y_start;
+	double		x_stop, y_stop;
+} GogAxisBaseView;
 
-typedef GogView		GogAxisBaseView;
 typedef GogViewClass	GogAxisBaseViewClass;
 
 #define GOG_AXIS_BASE_VIEW_TYPE		(gog_axis_base_view_get_type ())
