@@ -28,19 +28,20 @@
 #include <goffice/graph/gog-renderer.h>
 #include <goffice/graph/gog-style.h>
 #include <goffice/graph/gog-theme.h>
-
-#include <goffice/gtk/goffice-gtk.h>
-
 #include <goffice/utils/go-math.h>
 
 #include <gsf/gsf-impl-utils.h>
 
+#ifdef GOFFICE_WITH_GTK
+#include <goffice/gtk/goffice-gtk.h>
+#include <goffice/gtk/goffice-gtk.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkcelllayout.h>
 #include <gtk/gtkcombobox.h>
 #include <gtk/gtkliststore.h>
 #include <gtk/gtkspinbutton.h>
 #include <gtk/gtktogglebutton.h>
+#endif
 
 #include <glib/gi18n-lib.h>
 
@@ -334,6 +335,7 @@ gog_axis_base_set_position (GogAxisBase *axis_base, GogAxisPosition position)
 	axis_base->position = position;
 }
 
+#ifdef GOFFICE_WITH_GTK
 typedef struct {
 	GogAxisBase 	*axis_base;
 	GladeXML 	*gui;
@@ -552,6 +554,7 @@ gog_axis_base_populate_editor (GogObject *gobj,
 
 	(GOG_OBJECT_CLASS(gab_parent_klass)->populate_editor) (gobj, editor, dalloc, cc);
 }
+#endif
 
 static void
 gog_axis_base_init_style (GogStyledObject *gso, GogStyle *style)
@@ -612,7 +615,9 @@ gog_axis_base_class_init (GObjectClass *gobject_klass)
 			"Which axis to cross",
 			0, G_MAXUINT, 0, G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 
+#ifdef GOFFICE_WITH_GTK
 	gog_klass->populate_editor	= gog_axis_base_populate_editor;
+#endif
 	gog_klass->view_type		= gog_axis_base_view_get_type ();
 	gso_klass->init_style 		= gog_axis_base_init_style;
 }
@@ -659,6 +664,168 @@ GSF_CLASS_ABSTRACT (GogAxisBase, gog_axis_base,
 		    gog_axis_base_class_init, gog_axis_base_init,
 		    GOG_STYLED_OBJECT_TYPE);
 
+/************************************************************************/
+static gboolean gog_axis_base_view_point (GogView *view, double x, double y);
+
+static gboolean
+gog_tool_bound_is_valid_axis (GogView *view) 
+{
+	GogAxisBase *axis_base = GOG_AXIS_BASE (view->model);
+	GogAxisType type = gog_axis_get_atype (axis_base->axis);
+
+	return (type == GOG_AXIS_X ||
+		type == GOG_AXIS_Y);
+
+}
+
+#ifdef GOFFICE_WITH_GTK
+
+static gboolean
+gog_tool_select_axis_point (GogView *view, double x, double y, GogObject **gobj)
+{
+	return (gog_tool_bound_is_valid_axis (view) && 
+		gog_axis_base_view_point (view, x, y));
+}
+
+static void
+gog_tool_select_axis_render (GogView *view)
+{
+	if (gog_tool_bound_is_valid_axis (view)) {
+		ArtVpath *path = gog_renderer_get_rectangle_vpath (&view->allocation);
+		gog_renderer_draw_sharp_path (view->renderer, path);
+		art_free (path);
+	}
+}
+
+static GogTool gog_axis_tool_select_axis = {
+	N_("Select axis"),
+	GDK_LEFT_PTR,
+	gog_tool_select_axis_point, 
+	gog_tool_select_axis_render,
+	NULL /* init */,
+        NULL /* move */,	
+	NULL /* double-click */,
+	NULL /* destroy */
+};
+
+
+typedef struct {
+	GogAxis *axis;
+	GogAxisMap *map;
+	double length;
+	double start, stop;
+} MoveBoundData;
+	
+static gboolean
+gog_tool_move_start_bound_point (GogView *view, double x, double y, GogObject **gobj)
+{
+	return gog_tool_bound_is_valid_axis (view) &&
+		gog_renderer_in_grip (x, y, 
+				      GOG_AXIS_BASE_VIEW (view)->x_start, 
+				      GOG_AXIS_BASE_VIEW (view)->y_start);
+}
+
+static void
+gog_tool_move_start_bound_render (GogView *view)
+{
+	if (gog_tool_bound_is_valid_axis (view)) 
+		gog_renderer_draw_grip (view->renderer, 
+					GOG_AXIS_BASE_VIEW (view)->x_start, 
+					GOG_AXIS_BASE_VIEW (view)->y_start);
+}
+
+static gboolean
+gog_tool_move_stop_bound_point (GogView *view, double x, double y, GogObject **gobj)
+{
+	return (gog_tool_bound_is_valid_axis (view) && 
+		gog_renderer_in_grip (x, y, 
+				     GOG_AXIS_BASE_VIEW (view)->x_stop, 
+				     GOG_AXIS_BASE_VIEW (view)->y_stop));
+}
+
+static void
+gog_tool_move_stop_bound_render (GogView *view)
+{
+	if (gog_tool_bound_is_valid_axis (view)) 
+		gog_renderer_draw_grip (view->renderer, 
+					GOG_AXIS_BASE_VIEW (view)->x_stop, 
+					GOG_AXIS_BASE_VIEW (view)->y_stop);
+}
+
+static void
+gog_tool_move_bound_init (GogToolAction *action)
+{
+	MoveBoundData *data = g_new0 (MoveBoundData, 1);
+	GogAxisBaseView *view = GOG_AXIS_BASE_VIEW (action->view);
+	GogAxisBase *axis_base = GOG_AXIS_BASE (action->view->model);
+
+	action->data = data;
+	data->map = gog_axis_map_new (axis_base->axis, 0.0, 1.0);
+	data->axis = axis_base->axis;
+	data->length = hypot (view->x_start - view->x_stop,
+			      view->y_start - view->y_stop);
+	gog_axis_map_get_extents (data->map, &data->start, &data->stop);
+}
+
+static void
+gog_tool_move_start_bound_move (GogToolAction *action, double x, double y)
+{	
+	GogAxisBaseView *view = GOG_AXIS_BASE_VIEW (action->view);
+	MoveBoundData *data = action->data;
+	double a = 1.0 - MIN (((x - view->x_start) * (view->x_stop - view->x_start) +
+			       (y - view->y_start) * (view->y_stop - view->y_start)) / 
+			      (data->length * data->length), 0.9);
+
+	gog_axis_set_extents (data->axis, 
+			      data->stop + ((data->start - data->stop) / a),
+			      go_nan);
+}
+
+static void
+gog_tool_move_stop_bound_move (GogToolAction *action, double x, double y)
+{	
+	GogAxisBaseView *view = GOG_AXIS_BASE_VIEW (action->view);
+	MoveBoundData *data = action->data;
+	double a = 1.0 - MIN (((x - view->x_stop) * (view->x_start - view->x_stop) +
+			       (y - view->y_stop) * (view->y_start - view->y_stop)) / 
+			      (data->length * data->length), 0.9);
+
+	gog_axis_set_extents (data->axis, 
+			      go_nan, 
+			      data->start + ((data->stop - data->start) / a));
+}
+
+static void
+gog_tool_move_bound_destroy (GogToolAction *action) 
+{
+	MoveBoundData *data = action->data;
+
+	gog_axis_map_free (data->map);
+}
+
+static GogTool gog_axis_tool_start_bound = {
+	N_("Set start bound"), 
+	GDK_CROSS,
+	gog_tool_move_start_bound_point, 
+	gog_tool_move_start_bound_render,
+	gog_tool_move_bound_init, 
+	gog_tool_move_start_bound_move, 
+	NULL /* double-click */,
+	gog_tool_move_bound_destroy
+};
+
+static GogTool gog_axis_tool_stop_bound = {
+	N_("Set stop bound"), 
+	GDK_CROSS,
+	gog_tool_move_stop_bound_point, 
+	gog_tool_move_stop_bound_render,
+	gog_tool_move_bound_init, 
+	gog_tool_move_stop_bound_move, 
+	NULL /* double-click */,
+	gog_tool_move_bound_destroy
+};
+#endif
+
 /************************************************************************/
 
 #define POINT_MIN_DISTANCE 	5 	/* distance minimum between point and axis for point = TRUE, in pixels */
@@ -1108,6 +1275,7 @@ x_process (GogAxisBaseAction action, GogView *view, GogViewPadding *padding,
 	   GogViewAllocation const *plot_area, double x, double y)
 {
 	GogAxisBase *axis_base = GOG_AXIS_BASE (view->model);
+	GogAxisBaseView *aview = GOG_AXIS_BASE_VIEW (view);
 	GogAxisType axis_type = gog_axis_get_atype (axis_base->axis);
 	GogChartMap *c_map;
 	GogAxisMap *a_map;
@@ -1127,6 +1295,11 @@ x_process (GogAxisBaseAction action, GogView *view, GogViewPadding *padding,
 
 	gog_chart_map_free (c_map);
 
+	aview->x_start = ax;
+	aview->y_start = ay;
+	aview->x_stop = bx;
+	aview->y_stop = by;	
+
 	switch (action) {
 		case GOG_AXIS_BASE_RENDER:
 			axis_line_render (GOG_AXIS_BASE (view->model),
@@ -1159,6 +1332,7 @@ xy_process (GogAxisBaseAction action, GogView *view, GogViewPadding *padding,
 	    GogViewAllocation const *plot_area, double x, double y)
 {
 	GogAxisBase *axis_base = GOG_AXIS_BASE (view->model);
+	GogAxisBaseView *aview = GOG_AXIS_BASE_VIEW (view);
 	GogAxis *cross_axis;
 	GogChartMap *c_map;
 	GogAxisMap *a_map;
@@ -1209,6 +1383,11 @@ xy_process (GogAxisBaseAction action, GogView *view, GogViewPadding *padding,
 	}
 	gog_chart_map_free (c_map);
 
+	aview->x_start = ax;
+	aview->y_start = ay;
+	aview->x_stop = bx;
+	aview->y_stop = by;	
+
 	switch (action) {
 		case GOG_AXIS_BASE_RENDER:
 			axis_line_render (GOG_AXIS_BASE (view->model),
@@ -1355,9 +1534,7 @@ radar_process (GogAxisBaseAction action, GogView *view, GogViewPadding *padding,
 }
 
 static gboolean
-gog_axis_base_view_info_at_point (GogView *view, double x, double y,
-				  GogObject const *cur_selection,
-				  GogObject **obj, char **name)
+gog_axis_base_view_point (GogView *view, double x, double y)
 {
 	GogAxisBase *axis_base = GOG_AXIS_BASE (view->model);
 	GogAxisSet axis_set = gog_chart_get_axis_set (axis_base->chart);
@@ -1387,20 +1564,12 @@ gog_axis_base_view_info_at_point (GogView *view, double x, double y,
 		case GOG_AXIS_SET_UNKNOWN:
 			break;
 		default:
-			g_warning ("[AxisBaseView::info_at_point] not implemented for this axis set (%i)",
+			g_warning ("[AxisBaseView::point] not implemented for this axis set (%i)",
 				   axis_set);
 			break;
 	}
 
-	if (pointed) {
-		if (obj != NULL)
-			*obj = view->model;
-		if (name != NULL)
-			*name = NULL;
-		return TRUE;
-	}
-
-	return FALSE;
+	return pointed;
 }
 
 static void
@@ -1481,6 +1650,16 @@ gog_axis_base_view_render (GogView *view, GogViewAllocation const *bbox)
 	gog_renderer_pop_style (view->renderer);
 }
 
+static void
+gog_axis_base_build_toolkit (GogView *view)
+{
+#ifdef GOFFICE_WITH_GTK
+	view->toolkit = g_slist_prepend (view->toolkit, &gog_axis_tool_select_axis);
+	view->toolkit = g_slist_prepend (view->toolkit, &gog_axis_tool_start_bound);
+	view->toolkit = g_slist_prepend (view->toolkit, &gog_axis_tool_stop_bound);
+#endif
+}
+
 static void
 gog_axis_base_view_class_init (GogAxisBaseViewClass *gview_klass)
 {
@@ -1488,9 +1667,9 @@ gog_axis_base_view_class_init (GogAxisBaseViewClass *gview_klass)
 
 	gab_view_parent_klass = g_type_class_peek_parent (gview_klass);
 
-	view_klass->info_at_point	= gog_axis_base_view_info_at_point;
 	view_klass->padding_request 	= gog_axis_base_view_padding_request;
 	view_klass->render 		= gog_axis_base_view_render;
+	view_klass->build_toolkit	= gog_axis_base_build_toolkit;
 }
 
 GSF_CLASS (GogAxisBaseView, gog_axis_base_view,
