@@ -30,7 +30,6 @@
 #include <goffice/graph/gog-grid.h>
 #include <goffice/graph/gog-grid-line.h>
 #include <goffice/graph/gog-renderer.h>
-#include <goffice/gtk/goffice-gtk.h>
 #include <goffice/utils/go-math.h>
 
 #include <gsf/gsf-impl-utils.h>
@@ -38,10 +37,13 @@
 #include <string.h>
 #include <math.h>
 
+#ifdef GOFFICE_WITH_GTK
+#include <goffice/gtk/goffice-gtk.h>
 #include <gtk/gtkspinbutton.h>
 #include <gtk/gtktogglebutton.h>
+#endif
 
-const struct {
+static const struct {
 	char const *name;
 	GogAxisSet const axis_set;
 } axis_set_desc[] = {
@@ -74,6 +76,18 @@ gog_axis_set_from_str (char const *str)
 	return axis_set;
 }
 
+/*****************************************************************************/
+
+struct _GogChartMap {
+	GogChart 		*chart;
+	GogViewAllocation	 area;
+	gpointer	 	 data;
+	GogAxisMap		*axis_map[3];
+	gboolean		 is_valid;
+
+	void (*map_2D_to_view)	(GogChartMap *map, double x, double y, double *u, double *v);
+};
+
 static void
 calc_polygon_parameters (GogViewAllocation const *area, GogChartMapPolarData *data, gboolean fill_area)
 {
@@ -187,12 +201,36 @@ polar_map_2D_to_view (GogChartMap *map, double x, double y, double *u, double *v
 	*v = data->cy + r * data->ry * sin (t);		
 }
 
+/**
+ * gog_chart_map_get_polar_parms:
+ * @map: a #GogChartMap
+ *
+ * Convenience function for retrieving data related to polar plot layout.
+ *
+ * returns: a #GogChartMapPolarData struct.
+ **/
+
 GogChartMapPolarData *
 gog_chart_map_get_polar_parms (GogChartMap *map)
 {
 	return (GogChartMapPolarData *) map->data;
 }	
 
+/**
+ * gog_chart_map_new:
+ * @chart: a #GogChart
+ * @area: area allocated to chart
+ * @axis0: 1st dimension axis
+ * @axis1: 2nd dimension axis
+ * @axis2: 3rd dimension axis
+ * @fill_area: does chart fill allocated area
+ *
+ * Creates a new #GogChartMap, used for conversion from data space 
+ * to canvas space.
+ *
+ * returns: a new #GogChart object. 
+ **/
+
 GogChartMap *
 gog_chart_map_new (GogChart *chart, GogViewAllocation const *area, 
 		   GogAxis *axis0, GogAxis *axis1, GogAxis *axis2,
@@ -201,7 +239,7 @@ gog_chart_map_new (GogChart *chart, GogViewAllocation const *area,
 	GogChartMap *map;
 	GogAxisSet axis_set;
 
-	g_return_val_if_fail (GOG_CHART (chart) != NULL, NULL);
+	g_return_val_if_fail (IS_GOG_CHART (chart), NULL);
 
 	map = g_new (GogChartMap, 1);
 
@@ -292,14 +330,37 @@ gog_chart_map_new (GogChart *chart, GogViewAllocation const *area,
 	return map;
 }
 
+/**
+ * gog_chart_map_2D_to_view:
+ * @map: a #GogChartMap
+ * @x: data x value
+ * @y: data y value
+ * @u: placeholder for x converted value
+ * @v: placeholder for y converted value
+ *
+ * Converts a 2D coordinate from data space to canvas space.
+ **/
+
 void
 gog_chart_map_2D_to_view (GogChartMap *map, double x, double y, double *u, double *v)
 {
 	return (map->map_2D_to_view) (map, x, y, u, v);
 }
 
+/**
+ * gog_chart_map_get_axis_map:
+ * @map: a #GogChartMap
+ * @index: axis index
+ *
+ * Convenience function which returns one of the associated axis_map.
+ *
+ * Valid values are in range [0..2].
+ *
+ * returns: a #GogAxisMap.
+ **/
+
 GogAxisMap *
-gog_chart_map_get_axis_map (GogChartMap *map, unsigned i)
+gog_chart_map_get_axis_map (GogChartMap *map, unsigned int i)
 {
 	g_return_val_if_fail (map != NULL, NULL);
 	g_return_val_if_fail (i < 3, NULL);
@@ -307,6 +368,18 @@ gog_chart_map_get_axis_map (GogChartMap *map, unsigned i)
 	return map->axis_map[i];
 }
 
+/**
+ * gog_chart_map_is_valid:
+ * @map: a #GogChartMap
+ *
+ * Tests if @map was correctly initializied, i.e. if all associated axis_map
+ * are valid (see gog_axis_map_is_valid() ).
+ *
+ * given 
+ * to gog_chart_map_new. 
+ * returns: %TRUE if @map is valid.
+ **/
+
 gboolean
 gog_chart_map_is_valid (GogChartMap *map)
 {
@@ -315,6 +388,13 @@ gog_chart_map_is_valid (GogChartMap *map)
 	return map->is_valid;
 }
 
+/**
+ * gog_chart_map_free:
+ * @map: a #GogChartMap
+ *
+ * Frees @map object.
+ **/
+
 void
 gog_chart_map_free (GogChartMap *map)
 {
@@ -430,129 +510,16 @@ gog_chart_get_property (GObject *obj, guint param_id,
 	}
 }
 
-typedef struct {
-	GtkWidget	*x_spin, *y_spin, *w_spin, *h_spin;
-	gulong		w_spin_signal, h_spin_signal;
-	GtkWidget	*manual_toggle;
-	GogChart 	*chart;
-	GladeXML	*gui;
-} ChartPrefState;
-
-static void
-chart_pref_state_free (ChartPrefState *state) 
-{
-	g_object_unref (state->chart);
-	g_object_unref (state->gui);
-}
-
-static void
-cb_plot_area_changed (GtkWidget *spin, ChartPrefState *state)
-{
-	GogViewAllocation pos;
-	double value;
-	double max;
-
-       	value = gtk_spin_button_get_value (GTK_SPIN_BUTTON (spin)) / 100.0;
-
-       	gog_chart_get_plot_area (state->chart, &pos);
-	if (spin == state->x_spin) {
-		pos.x = value;
-		max = 1.0 - pos.x;
-		g_signal_handler_block (state->w_spin, state->w_spin_signal);
-		gtk_spin_button_set_range (GTK_SPIN_BUTTON (state->w_spin), 0.0, max * 100.0);
-		if (pos.w > max) pos.w = max;
-		gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->w_spin), pos.w * 100.0);
-		g_signal_handler_unblock (state->w_spin, state->w_spin_signal);
-	}
-	else if (spin == state->y_spin) {
-		pos.y = value;
-		max = 1.0 - pos.y;
-		g_signal_handler_block (state->h_spin, state->h_spin_signal);
-		gtk_spin_button_set_range (GTK_SPIN_BUTTON (state->h_spin), 0.0, max * 100.0);
-		if (pos.h > max) pos.h = max;
-		gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->h_spin), pos.w * 100.0);
-		g_signal_handler_unblock (state->h_spin, state->h_spin_signal);
-	}
-	else if (spin == state->w_spin) {
-		pos.w = value;
-	}
-	else if (spin == state->h_spin) {
-		pos.h = value;
-	}
-	gog_chart_set_plot_area (state->chart, &pos);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (state->manual_toggle), TRUE);
-}
-
-static void
-cb_manual_toggle_changed (GtkToggleButton *button, ChartPrefState *state)
-{
-	gog_chart_set_plot_area (state->chart, 
-		gtk_toggle_button_get_active (button) ?
-		&state->chart->plot_area : NULL);
-}
-
 static void
 gog_chart_populate_editor (GogObject *gobj, 
 			   GogEditor *editor, 
 			   G_GNUC_UNUSED GogDataAllocator *dalloc, 
 			   GOCmdContext *cc)
 {
-	GogChart *chart = GOG_CHART (gobj);
-	GtkWidget *w;
-	GladeXML *gui;
-	ChartPrefState *state;
 	static guint chart_pref_page = 0;
 	
-	gui = go_libglade_new ("gog-chart-prefs.glade", "gog_chart_prefs", GETTEXT_PACKAGE, cc);
-	if (gui == NULL)
-		return;
-
-	state = g_new (ChartPrefState, 1);
-	state->chart = chart;
-	state->gui = gui;
-	g_object_ref (G_OBJECT (gobj));
-
-	state->x_spin = glade_xml_get_widget (gui, "x_spin");
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->x_spin), 
-				   chart->plot_area.x * 100.0); 
-	g_signal_connect (G_OBJECT (state->x_spin), "value-changed", 
-			  G_CALLBACK (cb_plot_area_changed), state);
-
-	state->y_spin = glade_xml_get_widget (gui, "y_spin");
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->y_spin), 
-				   chart->plot_area.y * 100.0); 
-	g_signal_connect (G_OBJECT (state->y_spin), "value-changed", 
-			  G_CALLBACK (cb_plot_area_changed), state);
-
-	state->w_spin = glade_xml_get_widget (gui, "w_spin");
-	gtk_spin_button_set_range (GTK_SPIN_BUTTON (state->w_spin), 
-				   0.0, (1.0 - chart->plot_area.x) * 100.0);
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->w_spin), 
-				   100.0 * chart->plot_area.w); 
-	state->w_spin_signal = g_signal_connect (G_OBJECT (state->w_spin), "value-changed", 
-						 G_CALLBACK (cb_plot_area_changed), state);
-
-	state->h_spin = glade_xml_get_widget (gui, "h_spin");
-	gtk_spin_button_set_range (GTK_SPIN_BUTTON (state->h_spin), 
-				   0.0, (1.0 - chart->plot_area.y) * 100.0);
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->h_spin), 
-				   100.0 * chart->plot_area.h); 
-	state->h_spin_signal = g_signal_connect (G_OBJECT (state->h_spin), "value-changed", 
-						 G_CALLBACK (cb_plot_area_changed), state);
-
-	state->manual_toggle = glade_xml_get_widget (gui, "manual_toggle");
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (state->manual_toggle), 
-				      chart->is_plot_area_manual);
-	g_signal_connect (G_OBJECT (state->manual_toggle), "toggled", 
-			  G_CALLBACK (cb_manual_toggle_changed), state);
-
 	(GOG_OBJECT_CLASS(chart_parent_klass)->populate_editor) (gobj, editor, dalloc, cc);
 
-	w = glade_xml_get_widget (gui, "gog_chart_prefs");
-	g_object_set_data_full (G_OBJECT (w), "state", state, 
-				(GDestroyNotify) chart_pref_state_free);  
-	gog_editor_add_page (editor, w, _("Plot area"));
-	
 	gog_editor_set_store_page (editor, &chart_pref_page);
 }
 
@@ -745,7 +712,7 @@ gog_chart_class_init (GogObjectClass *gog_klass)
 
 	g_object_class_install_property (gobject_klass, CHART_PROP_CARDINALITY_VALID,
 		g_param_spec_boolean ("cardinality-valid", "cardinality-valid",
-				      "Is the charts cardinality currently vaid",
+				      "Is the charts cardinality currently valid",
 				      FALSE, G_PARAM_READABLE));
 	g_object_class_install_property (gobject_klass, CHART_PROP_PLOT_AREA,
 		g_param_spec_string ("plot-area", "Plot area",
@@ -793,7 +760,7 @@ GSF_CLASS (GogChart, gog_chart,
  * @cols :
  * @rows :
  *
- * Returns TRUE if the chart has been positioned.
+ * Returns: TRUE if the chart has been positioned.
  **/
 gboolean
 gog_chart_get_position (GogChart const *chart,
@@ -814,18 +781,18 @@ gog_chart_get_position (GogChart const *chart,
 
 /**
  * gog_chart_set_position :
- * @chart : #GogChart
- * @x :
- * @y :
- * @cols :
- * @rows :
+ * @chart: #GogChart
+ * @x:
+ * @y:
+ * @cols:
+ * @rows:
  *
  **/
 void
 gog_chart_set_position (GogChart *chart,
-			unsigned x, unsigned y, unsigned cols, unsigned rows)
+			unsigned int x, unsigned int y, unsigned int cols, unsigned int rows)
 {
-	g_return_if_fail (GOG_CHART (chart) != NULL);
+	g_return_if_fail (IS_GOG_CHART (chart));
 
 	if (chart->x == x && chart->y == y &&
 	    chart->cols == cols && chart->rows == rows)
@@ -885,7 +852,7 @@ gog_chart_get_cardinality (GogChart *chart, unsigned *full, unsigned *visible)
 	GSList *ptr;
 	unsigned tmp_full, tmp_visible;
 
-	g_return_if_fail (GOG_CHART (chart) != NULL);
+	g_return_if_fail (IS_GOG_CHART (chart));
 
 	if (!chart->cardinality_valid) {
 		chart->cardinality_valid = TRUE;
@@ -906,7 +873,7 @@ gog_chart_get_cardinality (GogChart *chart, unsigned *full, unsigned *visible)
 void
 gog_chart_request_cardinality_update (GogChart *chart)
 {
-	g_return_if_fail (GOG_CHART (chart) != NULL);
+	g_return_if_fail (IS_GOG_CHART (chart));
 	
 	if (chart->cardinality_valid) {
 		chart->cardinality_valid = FALSE;
@@ -920,7 +887,7 @@ gog_chart_foreach_elem (GogChart *chart, gboolean only_visible,
 {
 	GSList *ptr;
 
-	g_return_if_fail (GOG_CHART (chart) != NULL);
+	g_return_if_fail (IS_GOG_CHART (chart));
 	g_return_if_fail (chart->cardinality_valid);
 
 	for (ptr = chart->plots ; ptr != NULL ; ptr = ptr->next)
@@ -930,14 +897,14 @@ gog_chart_foreach_elem (GogChart *chart, gboolean only_visible,
 GSList *
 gog_chart_get_plots (GogChart const *chart)
 {
-	g_return_val_if_fail (GOG_CHART (chart) != NULL, NULL);
+	g_return_val_if_fail (IS_GOG_CHART (chart), NULL);
 	return chart->plots;
 }
 
 GogAxisSet
 gog_chart_get_axis_set (GogChart const *chart)
 {
-	g_return_val_if_fail (GOG_CHART (chart) != NULL, GOG_AXIS_SET_UNKNOWN);
+	g_return_val_if_fail (IS_GOG_CHART (chart), GOG_AXIS_SET_UNKNOWN);
 	return chart->axis_set;
 }
 
@@ -946,7 +913,7 @@ gog_chart_axis_set_is_valid (GogChart const *chart, GogAxisSet type)
 {
 	GSList *ptr;
 
-	g_return_val_if_fail (GOG_CHART (chart) != NULL, FALSE);
+	g_return_val_if_fail (IS_GOG_CHART (chart), FALSE);
 
 	for (ptr = chart->plots ; ptr != NULL ; ptr = ptr->next)
 		if (!gog_plot_axis_set_is_valid (ptr->data, type))
@@ -973,7 +940,7 @@ gog_chart_axis_set_assign (GogChart *chart, GogAxisSet axis_set)
 	GSList  *ptr;
 	GogAxisType type;
 
-	g_return_val_if_fail (GOG_CHART (chart) != NULL, FALSE);
+	g_return_val_if_fail (IS_GOG_CHART (chart), FALSE);
 
 	if (chart->axis_set == axis_set)
 		return TRUE;
@@ -1024,7 +991,7 @@ gog_chart_axis_set_assign (GogChart *chart, GogAxisSet axis_set)
  * @chart : #GogChart
  * @target  : #GogAxisType
  *
- * Return a list which the caller must free of all axis of type @target
+ * Returns: a list which the caller must free of all axis of type @target
  * associated with @chart.
  **/
 GSList *
@@ -1034,7 +1001,7 @@ gog_chart_get_axes (GogChart const *chart, GogAxisType target)
 	GogAxis *axis;
 	int type;
 
-	g_return_val_if_fail (GOG_CHART (chart) != NULL, NULL);
+	g_return_val_if_fail (IS_GOG_CHART (chart), NULL);
 
 	for (ptr = GOG_OBJECT (chart)->children ; ptr != NULL ; ptr = ptr->next) {
 		axis = ptr->data;
@@ -1055,15 +1022,15 @@ gog_chart_get_axes (GogChart const *chart, GogAxisType target)
 
 /**
  * gog_chart_get_grid :
- * @chart : #GogChart
+ * @chart: #GogChart
  *
- * Returns the grid associated with @chart if one exists
+ * Returns: the grid associated with @chart if one exists
  * otherwise NULL.
  **/
 GogGrid  *
 gog_chart_get_grid (GogChart const *chart)
 {
-	g_return_val_if_fail (GOG_CHART (chart) != NULL, NULL);
+	g_return_val_if_fail (IS_GOG_CHART (chart), NULL);
 	return GOG_GRID (chart->grid);
 }
 
@@ -1082,16 +1049,6 @@ typedef GogOutlinedViewClass	GogChartViewClass;
 
 static GogViewClass *cview_parent_klass;
 
-GogViewAllocation const *
-gog_chart_view_get_plot_area (GogView const *view)
-{
-	GogChartView *chart_view = GOG_CHART_VIEW (view);
-
-	g_return_val_if_fail ((GOG_CHART_VIEW (view) != NULL), NULL);
-
-	return & (chart_view->plot_area);
-}
-
 static void
 gog_chart_view_size_allocate (GogView *view, GogViewAllocation const *bbox)
 {
@@ -1227,12 +1184,21 @@ gog_chart_view_class_init (GogChartViewClass *gview_klass)
 	GogOutlinedViewClass *oview_klass = (GogOutlinedViewClass *) gview_klass;
 
 	cview_parent_klass = g_type_class_peek_parent (gview_klass);
-	view_klass->size_allocate   = gog_chart_view_size_allocate;
-	view_klass->clip = FALSE;
-	view_klass->render = gog_chart_view_render;
+
+	view_klass->size_allocate   	= gog_chart_view_size_allocate;
+	view_klass->render 		= gog_chart_view_render;
+	view_klass->clip 		= FALSE;
 	oview_klass->call_parent_render = FALSE;
 }
 
 static GSF_CLASS (GogChartView, gog_chart_view,
 		  gog_chart_view_class_init, gog_chart_view_init,
 		  GOG_OUTLINED_VIEW_TYPE)
+
+GogViewAllocation const *
+gog_chart_view_get_plot_area (GogView const *view)
+{
+	g_return_val_if_fail (IS_GOG_CHART_VIEW (view), NULL);
+
+	return &(GOG_CHART_VIEW (view)->plot_area);
+}
