@@ -22,14 +22,17 @@
 #include <goffice/goffice-config.h>
 #include <goffice/graph/gog-graph-impl.h>
 #include <goffice/graph/gog-chart-impl.h>
-#include <goffice/graph/gog-view.h>
 #include <goffice/graph/gog-renderer.h>
+#ifdef GOFFICE_WITH_CAIRO
+#include <goffice/graph/gog-renderer-cairo.h>
+#endif
 #include <goffice/graph/gog-style.h>
 #include <goffice/graph/gog-theme.h>
 #include <goffice/data/go-data.h>
 #include <goffice/utils/go-units.h>
 
 #include <gsf/gsf-impl-utils.h>
+
 #include <glib/gi18n-lib.h>
 #include <string.h>
 #include <stdlib.h>
@@ -50,6 +53,7 @@ enum {
 	GRAPH_REMOVE_DATA,
 	GRAPH_LAST_SIGNAL
 };
+
 static gulong gog_graph_signals [GRAPH_LAST_SIGNAL] = { 0, };
 static GObjectClass *graph_parent_klass;
 static GogViewClass *gview_parent_klass;
@@ -270,7 +274,7 @@ gog_graph_validate_chart_layout (GogGraph *graph)
 	unsigned i, max_col, max_row;
 	gboolean changed = FALSE;
 
-	g_return_val_if_fail (GOG_GRAPH (graph) != NULL, FALSE);
+	g_return_val_if_fail (IS_GOG_GRAPH (graph), FALSE);
 
 	/* There won't be many of charts so we do the right thing */
 
@@ -334,14 +338,14 @@ gog_graph_validate_chart_layout (GogGraph *graph)
 unsigned
 gog_graph_num_cols (GogGraph const *graph)
 {
-	g_return_val_if_fail (GOG_GRAPH (graph) != NULL, 1);
+	g_return_val_if_fail (IS_GOG_GRAPH (graph), 1);
 	return graph->num_cols;
 }
 
 unsigned
 gog_graph_num_rows (GogGraph const *graph)
 {
-	g_return_val_if_fail (GOG_GRAPH (graph) != NULL, 1);
+	g_return_val_if_fail (IS_GOG_GRAPH (graph), 1);
 	return graph->num_rows;
 }
 
@@ -361,29 +365,29 @@ gog_graph_dup (GogGraph const *graph)
 GogTheme *
 gog_graph_get_theme (GogGraph const *graph)
 {
-	g_return_val_if_fail (GOG_GRAPH (graph) != NULL, NULL);
+	g_return_val_if_fail (IS_GOG_GRAPH (graph), NULL);
 	return graph->theme;
 }
 
 void
 gog_graph_set_theme (GogGraph *graph, GogTheme *theme)
 {
-	g_return_if_fail (GOG_GRAPH (graph) != NULL);
-	g_return_if_fail (GOG_THEME (theme) != NULL);
+	g_return_if_fail (IS_GOG_GRAPH (graph));
+	g_return_if_fail (IS_GOG_THEME (theme));
 #warning TODO
 }
 
 /**
  * gog_graph_get_data :
- * @graph : #GogGraph
+ * @graph: #GogGraph
  *
- * Returns a list of the GOData objects that are data to the graph.
+ * Returns: a list of the GOData objects that are data to the graph.
  * The caller should _not_ modify or free the list.
  **/
 GSList *
 gog_graph_get_data (GogGraph const *graph)
 {
-	g_return_val_if_fail (GOG_GRAPH (graph) != NULL, NULL);
+	g_return_val_if_fail (IS_GOG_GRAPH (graph), NULL);
 	return graph->data;
 }
 
@@ -406,8 +410,8 @@ gog_graph_ref_data (GogGraph *graph, GOData *dat)
 	if (dat == NULL)
 		return NULL;
 
-	g_return_val_if_fail (GOG_GRAPH (graph) != NULL, dat);
-	g_return_val_if_fail (GO_DATA (dat) != NULL, dat);
+	g_return_val_if_fail (IS_GOG_GRAPH (graph), dat);
+	g_return_val_if_fail (IS_GO_DATA (dat), dat);
 
 	/* Does it already exist in the graph ? */
 	g_obj = G_OBJECT (graph);
@@ -452,14 +456,14 @@ gog_graph_unref_data (GogGraph *graph, GOData *dat)
 	if (dat == NULL)
 		return;
 
-	g_return_if_fail (GO_DATA (dat) != NULL);
+	g_return_if_fail (IS_GO_DATA (dat));
 
 	g_object_unref (dat);
 
 	if (graph == NULL)
 		return;
 
-	g_return_if_fail (GOG_GRAPH (graph) != NULL);
+	g_return_if_fail (IS_GOG_GRAPH (graph));
 
 	/* once we've been destroyed the list is gone */
 	if (graph->data == NULL)
@@ -506,7 +510,7 @@ gog_graph_request_update (GogGraph *graph)
 	if (G_OBJECT (graph)->ref_count <= 0)
 		return FALSE;
 
-	g_return_val_if_fail (GOG_GRAPH (graph) != NULL, FALSE);
+	g_return_val_if_fail (IS_GOG_GRAPH (graph), FALSE);
 
 	if (graph->idle_handler == 0) { /* higher priority than canvas */
 		graph->idle_handler = g_idle_add_full (G_PRIORITY_HIGH_IDLE,
@@ -533,17 +537,17 @@ gog_graph_force_update (GogGraph *graph)
 	}
 }
 /**
- * gog_graph_get_size:
+ * gog_graph_get_size :
  * @graph: #GogGraph
  * @width: logical width in pts
  * @height: logical height in pts
  *
- * Returns the logical size of graph, in points.
+ * FIXME Returns the logical size of graph, in points.
  **/
 void
 gog_graph_get_size (GogGraph *graph, double *width, double *height)
 {
-	g_return_if_fail (GOG_GRAPH (graph) != NULL);
+	g_return_if_fail (IS_GOG_GRAPH (graph));
 
 	if (width != NULL)
 		*width = graph->width;
@@ -561,7 +565,7 @@ gog_graph_get_size (GogGraph *graph, double *width, double *height)
 void
 gog_graph_set_size (GogGraph *graph, double width, double height)
 {
-	g_return_if_fail (GOG_GRAPH (graph) != NULL);
+	g_return_if_fail (IS_GOG_GRAPH (graph));
 
 	if (width != graph->width || height != graph->height) {
 		graph->height = height;
@@ -572,8 +576,30 @@ gog_graph_set_size (GogGraph *graph, double width, double height)
 
 /************************************************************************/
 
-typedef GogOutlinedView		GogGraphView;
-typedef GogOutlinedViewClass	GogGraphViewClass;
+enum {
+	GRAPH_VIEW_SELECTION_CHANGED,
+	GRAPH_VIEW_LAST_SIGNAL
+};
+
+static gulong gog_graph_view_signals [GRAPH_VIEW_LAST_SIGNAL] = { 0, };
+
+struct _GogGraphView {
+	GogOutlinedView		 base;
+
+	GogToolAction		*action;
+	GogObject		*selected_object;
+	GogView			*selected_view;
+#ifdef GOFFICE_WITH_GTK
+	GdkCursorType		 cursor_type;
+#endif
+};
+
+typedef struct {
+	GogOutlinedViewClass	base;
+
+	/* signals */
+	void (*selection_changed) (GogGraphView *view, GogObject *gobj);
+} GogGraphViewClass;
 
 #define GOG_GRAPH_VIEW_TYPE	(gog_graph_view_get_type ())
 #define GOG_GRAPH_VIEW(o)	(G_TYPE_CHECK_INSTANCE_CAST ((o), GOG_GRAPH_VIEW_TYPE, GogGraphView))
@@ -634,6 +660,33 @@ gog_graph_view_size_allocate (GogView *view, GogViewAllocation const *bbox)
 	}
 }
 
+static void
+gog_graph_view_render  (GogView *view, GogViewAllocation const *bbox)
+{
+	GogGraphView *gview = GOG_GRAPH_VIEW (view);
+
+	gview_parent_klass->render (view, bbox);
+
+	if (gview->selected_view != NULL) {
+		gog_renderer_push_selection_style (view->renderer);
+		gog_view_render_toolkit (gview->selected_view);
+		gog_renderer_pop_style (view->renderer);
+	}
+}
+
+static void
+gog_graph_view_finalize (GObject *obj)
+{
+	GogGraphView *gview = GOG_GRAPH_VIEW (obj);
+
+	if (gview->action != NULL) {
+		gog_tool_action_free (gview->action);
+		gview->action = NULL;
+	}
+	
+	(G_OBJECT_CLASS (gview_parent_klass)->finalize) (obj);
+}
+
 static void
 gog_graph_view_class_init (GogGraphViewClass *gview_klass)
 {
@@ -642,6 +695,9 @@ gog_graph_view_class_init (GogGraphViewClass *gview_klass)
 
 	gview_parent_klass = g_type_class_peek_parent (gview_klass);
 	gobject_klass->set_property = gog_graph_view_set_property;
+	gobject_klass->finalize	    = gog_graph_view_finalize;
+	view_klass->build_toolkit   = NULL;
+	view_klass->render	    = gog_graph_view_render;
 	view_klass->size_allocate   = gog_graph_view_size_allocate;
 	view_klass->clip	    = TRUE;
 
@@ -649,8 +705,244 @@ gog_graph_view_class_init (GogGraphViewClass *gview_klass)
 		g_param_spec_object ("renderer", "renderer",
 			"the renderer for this view",
 			GOG_RENDERER_TYPE, G_PARAM_WRITABLE));
+	
+	gog_graph_view_signals [GRAPH_VIEW_SELECTION_CHANGED] = g_signal_new ("selection-changed",
+		G_TYPE_FROM_CLASS (gview_klass),
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (GogGraphViewClass, selection_changed),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__OBJECT,
+		G_TYPE_NONE,
+		1, G_TYPE_OBJECT);
+}
+
+static void
+gog_graph_view_init (GogGraphView *view)
+{
+	view->action = NULL;
+	view->selected_object = NULL;
+	view->selected_view = NULL;
+#ifdef GOFFICE_WITH_GTK
+	view->cursor_type = GDK_LEFT_PTR;
+#endif
 }
 
 GSF_CLASS (GogGraphView, gog_graph_view,
-	   gog_graph_view_class_init, NULL,
+	   gog_graph_view_class_init, gog_graph_view_init,
 	   GOG_OUTLINED_VIEW_TYPE)
+
+#ifdef GOFFICE_WITH_GTK
+static void
+update_cursor (GogGraphView *view, GogTool *tool, GdkWindow *window)
+{
+	GdkCursor *cursor;
+	GdkCursorType cursor_type;
+	
+	cursor_type = tool != NULL ? tool->cursor_type : GDK_LEFT_PTR;
+	if (cursor_type != view->cursor_type) {
+		view->cursor_type = cursor_type;
+		/* FIXME: gdk_display_get_default is probably not a good idea */
+		cursor = gdk_cursor_new_for_display (gdk_display_get_default (), cursor_type);
+		gdk_window_set_cursor (window, cursor);
+		gdk_cursor_unref (cursor);
+	}
+}
+#endif
+
+static void
+update_action (GogGraphView *view, GogTool *tool, double x, double y)
+{
+	if (view->action != NULL) { 
+		gog_tool_action_free (view->action);
+		view->action = NULL;
+	}
+	
+	if (tool != NULL)
+		view->action = gog_tool_action_new (view->selected_view, tool, x, y);
+}
+
+#ifdef GOFFICE_WITH_GTK
+#include <gdk/gdkcursor.h>
+
+/**
+ * gog_graph_view_handle_event:
+ * @gview : #GogGraphView
+ * @event : #GdkEvent
+ *
+ * Handle events.
+ */
+void
+gog_graph_view_handle_event (GogGraphView *view, GdkEvent *event, 
+			     double x_offset, double y_offset)
+{
+	GogObject *old_object = view->selected_object;
+	GogTool *tool = NULL;
+	GdkWindow *window = event->any.window;
+	double x, y;
+	int x_int, y_int;
+	
+	x = event->button.x - x_offset;
+	y = event->button.y - y_offset;
+
+	switch (event->type) {
+		case GDK_2BUTTON_PRESS:
+			if (view->action != NULL) 
+				gog_tool_action_double_click (view->action);
+			break;
+		case GDK_BUTTON_PRESS: 
+			if (view->selected_view != NULL)
+				tool = gog_view_get_tool_at_point (view->selected_view, x, y,
+								   &view->selected_object);
+			if (tool == NULL) 
+				view->selected_view = gog_view_get_view_at_point (GOG_VIEW (view), x, y, 
+										  &view->selected_object, 
+										  &tool);
+			if (old_object != view->selected_object) {
+				g_signal_emit (G_OBJECT (view), 
+					       gog_graph_view_signals [GRAPH_VIEW_SELECTION_CHANGED],
+					       0, view->selected_object);
+				gog_view_queue_redraw (GOG_VIEW (view));
+			}
+			update_action (view, tool, x, y);
+			update_cursor (view, tool, window);
+			break;
+		case GDK_MOTION_NOTIFY: 
+			if (event->motion.is_hint) {
+				gdk_window_get_pointer (window, &x_int, &y_int, NULL);
+				x = x_int - x_offset;
+				y = y_int - y_offset;
+			}
+			if (view->action != NULL) {
+				gog_tool_action_move (view->action, x, y);
+				gdk_window_process_updates (window, TRUE);
+			}
+			else if (view->selected_view != NULL) {
+				tool = gog_view_get_tool_at_point (view->selected_view, x, y, NULL);
+				update_cursor (view, tool, window);
+			}
+			break;
+		case GDK_BUTTON_RELEASE:
+			update_action (view, NULL, 0, 0);
+			if (view->selected_view != NULL) {
+				tool = gog_view_get_tool_at_point (view->selected_view, x, y, NULL);
+				update_cursor (view, tool, window);
+				gog_object_request_editor_update (view->selected_view->model);
+			}
+			break;
+		default:
+			break;
+	}
+}
+#endif
+
+/**
+ * gog_graph_view_get_selection :
+ * @gview: #GogGraphView
+ *
+ * Returns: current selected view.
+ **/
+GogView *
+gog_graph_view_get_selection (GogGraphView *gview)
+{
+	g_return_val_if_fail (IS_GOG_GRAPH_VIEW (gview), NULL);
+	
+	return gview->selected_view;
+}
+
+/**
+ * gog_graph_view_set_selection
+ * @gview : #GogGraphView
+ * @gobj : new selected object
+ * 
+ * Sets @gobj as current selection. If @gobj is different from previously 
+ * selected object, a selection-changed signal is emitted.
+ **/
+void
+gog_graph_view_set_selection (GogGraphView *gview, GogObject *gobj)
+{
+	GogView *view;
+	
+	g_return_if_fail (IS_GOG_GRAPH_VIEW (gview));
+	g_return_if_fail (IS_GOG_OBJECT (gobj));
+
+	if (gview->selected_object == gobj)
+		return;
+
+	gview->selected_object = gobj;
+	view = gog_view_find_child_view (GOG_VIEW (gview), gobj);
+
+	if (gview->selected_view != view) {
+		gview->selected_view = view;
+		update_action (gview, NULL, 0, 0);
+	}
+
+	gog_view_queue_redraw (GOG_VIEW (gview));
+	g_signal_emit (G_OBJECT (gview), 
+		       gog_graph_view_signals [GRAPH_VIEW_SELECTION_CHANGED], 
+		       0, gobj);
+}
+
+/**
+ * gog_graph_get_supported_image_formats:
+ *
+ * Builds a list of supported formats for image export.
+ *
+ * returns: a #GSList of #GOImageFormat.
+ **/
+
+GSList *
+gog_graph_get_supported_image_formats (void)
+{
+	static GOImageFormat supported_formats[] = {
+#ifdef GOG_RENDERER_CAIRO_WITH_PS
+		GO_IMAGE_FORMAT_PS,
+#endif
+#ifdef GOG_RENDERER_CAIRO_WITH_PDF
+		GO_IMAGE_FORMAT_PDF,
+#endif
+		GO_IMAGE_FORMAT_JPG,
+		GO_IMAGE_FORMAT_PNG,
+		GO_IMAGE_FORMAT_SVG
+	};
+	GSList *list = NULL;
+	unsigned i;
+
+	for (i = 0; i < G_N_ELEMENTS (supported_formats); i++)
+		list = g_slist_prepend (list, GUINT_TO_POINTER (supported_formats[i]));
+
+	return list;
+}
+
+/**
+ * gog_graph_export_image:
+ * @graph: a #GogGraph
+ * @format: image format for export
+ * @output: a #GsfOutput stream
+ * @x_dpi: x resolution of exported graph
+ * @y_dpi: y resolution of exported graph
+ *
+ * Exports an image of @graph in given @format, writing results in a #GsfOutput stream.
+ * If export format type is a bitmap one, it computes image size with x_dpi, y_dpi and 
+ * @graph size (see @gog_graph_get_size()).
+ *
+ * returns: %TRUE if export succeed.
+ **/
+
+gboolean
+gog_graph_export_image (GogGraph *graph, GOImageFormat format, GsfOutput *output,
+			double x_dpi, double y_dpi)
+{
+	GogRenderer *renderer;
+	gboolean result;
+		
+	g_return_val_if_fail (IS_GOG_GRAPH (graph), FALSE);
+	g_return_val_if_fail (format != GO_IMAGE_FORMAT_UNKNOWN, FALSE);
+	
+	renderer = gog_renderer_new_for_format (graph, format);
+	g_return_val_if_fail (renderer != NULL, FALSE);
+
+	result = gog_renderer_export_image (renderer, format, output, x_dpi, y_dpi);
+	g_object_unref (renderer);
+
+	return result;
+}
