@@ -29,16 +29,21 @@
 #include <goffice/graph/gog-theme.h>
 #include <goffice/graph/gog-graph.h>
 #include <goffice/graph/gog-object-xml.h>
+#include <goffice/graph/gog-renderer.h>
 #include <goffice/data/go-data.h>
 #include <goffice/utils/go-math.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
-#include <gtk/gtktable.h>
+#ifdef GOFFICE_WITH_GTK
+#include <gtk/gtkcellrenderertext.h>
+#include <gtk/gtkcelllayout.h>
 #include <gtk/gtkcombobox.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkliststore.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkcelllayout.h>
+#include <gtk/gtkspinbutton.h>
+#include <gtk/gtktogglebutton.h>
+#include <gtk/gtktable.h>
+#endif
 
 #include <gsf/gsf-impl-utils.h>
 #include <string.h>
@@ -140,11 +145,81 @@ role_series_pre_remove (GogObject *parent, GogObject *series)
 	gog_plot_request_cardinality_update (plot);
 }
 
+#ifdef GOFFICE_WITH_GTK
 typedef struct {
+	GladeXML	*gui;
+	GtkWidget	*x_spin, *y_spin, *w_spin, *h_spin;
+	gulong		 w_spin_signal, h_spin_signal;
+	GtkWidget	*position_select_combo;
+	GtkWidget	*manual_setting_table;
+	GogChart	*chart;
 	GogPlot		*plot;
-	GogAxisType	type;
+	gulong		 update_editor_handler;
 } PlotPrefState;
 
+static void
+plot_pref_state_free (PlotPrefState *state) 
+{
+	g_signal_handler_disconnect (G_OBJECT (state->plot), state->update_editor_handler);
+	g_object_unref (state->plot);
+	g_object_unref (state->chart);
+	g_object_unref (state->gui);
+}
+
+static void
+cb_plot_area_changed (GtkWidget *spin, PlotPrefState *state)
+{
+	GogViewAllocation pos;
+	double value;
+	double max;
+
+       	value = gtk_spin_button_get_value (GTK_SPIN_BUTTON (spin)) / 100.0;
+
+       	gog_chart_get_plot_area (state->chart, &pos);
+	if (spin == state->x_spin) {
+		pos.x = value;
+		max = 1.0 - pos.x;
+		g_signal_handler_block (state->w_spin, state->w_spin_signal);
+		gtk_spin_button_set_range (GTK_SPIN_BUTTON (state->w_spin), 0.0, max * 100.0);
+		if (pos.w > max) pos.w = max;
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->w_spin), pos.w * 100.0);
+		g_signal_handler_unblock (state->w_spin, state->w_spin_signal);
+	}
+	else if (spin == state->y_spin) {
+		pos.y = value;
+		max = 1.0 - pos.y;
+		g_signal_handler_block (state->h_spin, state->h_spin_signal);
+		gtk_spin_button_set_range (GTK_SPIN_BUTTON (state->h_spin), 0.0, max * 100.0);
+		if (pos.h > max) pos.h = max;
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->h_spin), pos.w * 100.0);
+		g_signal_handler_unblock (state->h_spin, state->h_spin_signal);
+	}
+	else if (spin == state->w_spin) {
+		pos.w = value;
+	}
+	else if (spin == state->h_spin) {
+		pos.h = value;
+	}
+	gog_chart_set_plot_area (state->chart, &pos);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (state->position_select_combo), 1);
+	gtk_widget_show (state->manual_setting_table);
+}
+
+static void
+cb_manual_position_changed (GtkComboBox *combo, PlotPrefState *state)
+{
+	if (gtk_combo_box_get_active (combo) == 1) {
+		GogViewAllocation plot_area;
+
+		gog_chart_get_plot_area (state->chart, &plot_area);
+		gog_chart_set_plot_area (state->chart, &plot_area);
+		gtk_widget_show (state->manual_setting_table);
+	} else {
+		gog_chart_set_plot_area (state->chart, NULL);
+		gtk_widget_hide (state->manual_setting_table);
+	}
+}
+
 static void
 cb_axis_changed (GtkComboBox *combo, PlotPrefState *state)
 {
@@ -155,7 +230,35 @@ cb_axis_changed (GtkComboBox *combo, PlotPrefState *state)
 	memset (&value, 0, sizeof (GValue));
 	gtk_combo_box_get_active_iter (combo, &iter);
 	gtk_tree_model_get_value (model, &iter, 1, &value);
-	gog_plot_set_axis_by_id (state->plot, state->type, g_value_get_uint (&value));
+	gog_plot_set_axis_by_id (state->plot, 
+				 GPOINTER_TO_UINT (g_object_get_data (G_OBJECT(combo), "axis-type")),
+				 g_value_get_uint (&value));
+}
+
+static void
+cb_update_editor (GogObject *gobj, PlotPrefState *state)
+{
+	GogViewAllocation plot_area;
+	gboolean is_plot_area_manual;
+
+	is_plot_area_manual = gog_chart_get_plot_area (state->chart, &plot_area);
+	
+	if (state->x_spin != NULL)
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->x_spin), plot_area.x * 100.0);
+	if (state->y_spin != NULL)
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->y_spin), plot_area.y * 100.0);
+	if (state->w_spin != NULL)
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->w_spin), plot_area.w * 100.0);
+	if (state->h_spin != NULL)
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->h_spin), plot_area.h * 100.0);
+	if (state->position_select_combo != NULL) {
+		gtk_combo_box_set_active (GTK_COMBO_BOX (state->position_select_combo), 
+					  is_plot_area_manual ? 1 : 0);
+		if (is_plot_area_manual)
+			gtk_widget_show (state->manual_setting_table);
+		else
+			gtk_widget_hide (state->manual_setting_table);
+	}
 }
 
 static void
@@ -174,82 +277,140 @@ gog_plot_populate_editor (GogObject *obj,
 		N_("Pseudo 3D axis:")
 	};
 	
-	GtkWidget *table, *combo;
+	GtkWidget *w;
 	GogAxisType type;
 	GogPlot *plot = GOG_PLOT (obj);
 	unsigned count = 0, axis_count;
 	GSList *axes, *ptr;
 	GogChart *chart = GOG_CHART (gog_object_get_parent (obj));
 	GogAxis *axis;
+	GogViewAllocation plot_area;
 	GtkListStore *store;
 	GtkTreeIter iter;
 	GtkCellRenderer *cell;
+	GladeXML *gui;
 	PlotPrefState *state;
+	gboolean is_plot_area_manual;
 
 	g_return_if_fail (chart != NULL);
+	gui = go_libglade_new ("gog-plot-prefs.glade", "gog_plot_prefs", GETTEXT_PACKAGE, cc);
+	g_return_if_fail (gui != NULL);
 
-	if (gog_chart_get_axis_set (chart) != GOG_AXIS_SET_XY) {
-		(GOG_OBJECT_CLASS(plot_parent_klass)->populate_editor) (obj, editor, dalloc, cc);
-		return;
-	}
+	state = g_new  (PlotPrefState, 1);
+	state->plot = plot;
+	state->chart = chart;
+	state->gui = gui;
 
-	table = gtk_table_new (0, 1, FALSE);
-	for (type = 0 ; type < GOG_AXIS_TYPES ; type++) {
-		if (plot->axis[type] != NULL) {
-			count++;
-			gtk_table_resize (GTK_TABLE (table), count, 1);
-			gtk_table_attach (GTK_TABLE (table), gtk_label_new (_(axis_labels[type])),
-					  0, 1, count - 1, count, 0, 0, 0, 0);
+	g_object_ref (G_OBJECT (plot));
+	g_object_ref (G_OBJECT (chart));
 
-			store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_UINT);
-			combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (store));
+	if (gog_chart_get_axis_set (chart) == GOG_AXIS_SET_XY) {
+		GtkWidget *combo;
+		GtkWidget *table = gtk_table_new (0, 1, FALSE);
 
-			cell = gtk_cell_renderer_text_new ();
-			gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), cell, TRUE);
-			gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo), cell,
-							"text", 0,
-							NULL);
-			
-			axes = gog_chart_get_axes (chart, type);
-			axis_count = 0;
-			for (ptr = axes; ptr != NULL; ptr = ptr->next) {
-				axis = GOG_AXIS (ptr->data);
-				gtk_list_store_prepend (store, &iter);
-				gtk_list_store_set (store, &iter, 
-					0, gog_object_get_name (GOG_OBJECT (axis)), 
-					1, gog_object_get_id (GOG_OBJECT (axis)),
-					-1);
-				if (axis == plot->axis[type])
-					gtk_combo_box_set_active_iter (GTK_COMBO_BOX (combo), &iter); 
-				axis_count++;
+		for (type = 0 ; type < GOG_AXIS_TYPES ; type++) {
+			if (plot->axis[type] != NULL) {
+				count++;
+				gtk_table_resize (GTK_TABLE (table), count, 1);
+				gtk_table_attach (GTK_TABLE (table), gtk_label_new (_(axis_labels[type])),
+						  0, 1, count - 1, count, 0, 0, 0, 0);
+
+				store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_UINT);
+				combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (store));
+
+				cell = gtk_cell_renderer_text_new ();
+				gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), cell, TRUE);
+				gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo), cell,
+								"text", 0,
+								NULL);
+
+				axes = gog_chart_get_axes (chart, type);
+				axis_count = 0;
+				for (ptr = axes; ptr != NULL; ptr = ptr->next) {
+					axis = GOG_AXIS (ptr->data);
+					gtk_list_store_prepend (store, &iter);
+					gtk_list_store_set (store, &iter, 
+							    0, gog_object_get_name (GOG_OBJECT (axis)), 
+							    1, gog_object_get_id (GOG_OBJECT (axis)),
+							    -1);
+					if (axis == plot->axis[type])
+						gtk_combo_box_set_active_iter (GTK_COMBO_BOX (combo), &iter); 
+					axis_count++;
+				}
+				if (axis_count < 2)
+					gtk_widget_set_sensitive (GTK_WIDGET (combo), FALSE);
+				g_slist_free (axes);
+				gtk_table_attach (GTK_TABLE (table), combo,
+						  1, 2, count - 1, count, 0, 0, 0, 0);
+				g_object_set_data (G_OBJECT (combo), "axis-type", GUINT_TO_POINTER (type));
+				g_signal_connect (G_OBJECT (combo), "changed",
+						  G_CALLBACK (cb_axis_changed), state);
 			}
-			if (axis_count < 2)
-				gtk_widget_set_sensitive (GTK_WIDGET (combo), FALSE);
-			g_slist_free (axes);
-			gtk_table_attach (GTK_TABLE (table), combo,
-					  1, 2, count - 1, count, 0, 0, 0, 0);
-			state = g_new (PlotPrefState, 1);
-			state->plot = plot;
-			state->type = type;
-			g_signal_connect (G_OBJECT (combo), "changed",
-					  G_CALLBACK (cb_axis_changed), state);
-			g_object_set_data_full (G_OBJECT (combo),
-						"state", state, (GDestroyNotify) g_free);
 		}
-	}
 
-	if (count > 0) {
-		gtk_table_set_col_spacings (GTK_TABLE (table), 12);
-		gtk_table_set_row_spacings (GTK_TABLE (table), 6);
-		gtk_container_set_border_width (GTK_CONTAINER (table), 12);
-		gtk_widget_show_all (table);
-		gog_editor_add_page (editor, table, _("Axes"));
+		if (count > 0) {
+			gtk_table_set_col_spacings (GTK_TABLE (table), 12);
+			gtk_table_set_row_spacings (GTK_TABLE (table), 6);
+			gtk_container_set_border_width (GTK_CONTAINER (table), 12);
+			gtk_widget_show_all (table);
+			gog_editor_add_page (editor, table, _("Axes"));
+		}
+		else
+			g_object_unref (G_OBJECT (table));
 	}
-	else
-		g_object_unref (G_OBJECT (table));
 	
+	is_plot_area_manual = gog_chart_get_plot_area (chart, &plot_area);
+	
+	state->x_spin = glade_xml_get_widget (gui, "x_spin");
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->x_spin), 
+				   plot_area.x * 100.0); 
+	g_signal_connect (G_OBJECT (state->x_spin), "value-changed", 
+			  G_CALLBACK (cb_plot_area_changed), state);
+
+	state->y_spin = glade_xml_get_widget (gui, "y_spin");
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->y_spin), 
+				   plot_area.y * 100.0); 
+	g_signal_connect (G_OBJECT (state->y_spin), "value-changed", 
+			  G_CALLBACK (cb_plot_area_changed), state);
+
+	state->w_spin = glade_xml_get_widget (gui, "w_spin");
+	gtk_spin_button_set_range (GTK_SPIN_BUTTON (state->w_spin), 
+				   0.0, (1.0 - plot_area.x) * 100.0);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->w_spin), 
+				   100.0 * plot_area.w); 
+	state->w_spin_signal = g_signal_connect (G_OBJECT (state->w_spin), "value-changed", 
+						 G_CALLBACK (cb_plot_area_changed), state);
+
+	state->h_spin = glade_xml_get_widget (gui, "h_spin");
+	gtk_spin_button_set_range (GTK_SPIN_BUTTON (state->h_spin), 
+				   0.0, (1.0 - plot_area.y) * 100.0);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (state->h_spin), 
+				   100.0 * plot_area.h); 
+	state->h_spin_signal = g_signal_connect (G_OBJECT (state->h_spin), "value-changed", 
+						 G_CALLBACK (cb_plot_area_changed), state);
+
+	state->position_select_combo = glade_xml_get_widget (gui, "position_select_combo");
+	gtk_combo_box_set_active (GTK_COMBO_BOX (state->position_select_combo), 
+				  is_plot_area_manual ? 1 : 0); 
+	state->manual_setting_table = glade_xml_get_widget (gui, "manual_setting_table");
+	if (!is_plot_area_manual)
+		gtk_widget_hide (state->manual_setting_table);
+	
+	g_signal_connect (G_OBJECT (state->position_select_combo),
+			  "changed", G_CALLBACK (cb_manual_position_changed), state);
+
+	w = glade_xml_get_widget (gui, "gog_plot_prefs");
+	g_object_set_data_full (G_OBJECT (w), "state", state, 
+				(GDestroyNotify) plot_pref_state_free);  
+	gog_editor_add_page (editor, w, _("Plot area"));
+
+	state->update_editor_handler = g_signal_connect (G_OBJECT (plot), 
+							 "update-editor", 
+							 G_CALLBACK (cb_update_editor), state);
+
 	(GOG_OBJECT_CLASS(plot_parent_klass)->populate_editor) (obj, editor, dalloc, cc);
 }
+#endif
 
 static void
 gog_plot_set_property (GObject *obj, guint param_id,
@@ -352,7 +513,9 @@ gog_plot_class_init (GogObjectClass *gog_klass)
 	gobject_klass->finalize		= gog_plot_finalize;
 	gobject_klass->set_property	= gog_plot_set_property;
 	gobject_klass->get_property	= gog_plot_get_property;
+#ifdef GOFFICE_WITH_GTK
 	gog_klass->populate_editor	= gog_plot_populate_editor;
+#endif
 	plot_klass->axis_set 		= GOG_AXIS_SET_NONE;
 	plot_klass->guru_helper		= NULL;
 
@@ -375,7 +538,7 @@ gog_plot_class_init (GogObjectClass *gog_klass)
 			NULL, G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, PLOT_PROP_GURU_HINTS,
 		g_param_spec_string ("guru-hints", _("Guru hints"), 
-			_("Semicolon separated list of hints for automatic addition of objects in"
+			_("Semicolon separated list of hints for automatic addition of objects in "
 			  "guru dialog"),
 			NULL, G_PARAM_READWRITE));
 
@@ -427,8 +590,8 @@ gog_plot_new_by_type (GogPlotType const *type)
 gboolean
 gog_plot_make_similar (GogPlot *dst, GogPlot const *src)
 {
-	g_return_val_if_fail (GOG_PLOT (dst) != NULL, TRUE);
-	g_return_val_if_fail (GOG_PLOT (src) != NULL, TRUE);
+	g_return_val_if_fail (IS_GOG_PLOT (dst), TRUE);
+	g_return_val_if_fail (IS_GOG_PLOT (src), TRUE);
 
 	return FALSE;
 }
@@ -437,13 +600,17 @@ gog_plot_make_similar (GogPlot *dst, GogPlot const *src)
 GogSeries *
 gog_plot_new_series (GogPlot *plot)
 {
-	GogObject *res = gog_object_add_by_name (GOG_OBJECT (plot), "Series", NULL);
+	GogObject *res;
+
+	g_return_val_if_fail (IS_GOG_PLOT (plot), NULL);
+
+	res = gog_object_add_by_name (GOG_OBJECT (plot), "Series", NULL);
 	return res ? GOG_SERIES (res) : NULL;
 }
 GogPlotDesc const *
 gog_plot_description (GogPlot const *plot)
 {
-	g_return_val_if_fail (GOG_PLOT (plot) != NULL, NULL);
+	g_return_val_if_fail (IS_GOG_PLOT (plot), NULL);
 	return &plot->desc;
 }
 
@@ -457,7 +624,7 @@ gog_plot_get_chart (GogPlot const *plot)
 void
 gog_plot_request_cardinality_update (GogPlot *plot)
 {
-	g_return_if_fail (GOG_PLOT (plot) != NULL);
+	g_return_if_fail (IS_GOG_PLOT (plot));
 
 	if (plot->cardinality_valid) {
 		GogChart *chart = gog_plot_get_chart (plot);
@@ -478,7 +645,7 @@ gog_plot_request_cardinality_update (GogPlot *plot)
 void
 gog_plot_get_cardinality (GogPlot *plot, unsigned *full, unsigned *visible)
 {
-	g_return_if_fail (GOG_PLOT (plot) != NULL);
+	g_return_if_fail (IS_GOG_PLOT (plot));
 
 	if (!plot->cardinality_valid) {
 		GogSeries *series;
@@ -529,7 +696,7 @@ gog_plot_foreach_elem (GogPlot *plot, gboolean only_visible,
 	GogPlotClass *klass = GOG_PLOT_GET_CLASS (plot);
 	GList *overrides;
 
-	g_return_if_fail (GOG_PLOT (plot) != NULL);
+	g_return_if_fail (IS_GOG_PLOT (plot));
 	if (!plot->cardinality_valid)
 		gog_plot_get_cardinality (plot, NULL, NULL);
 
@@ -593,7 +760,7 @@ gog_plot_foreach_elem (GogPlot *plot, gboolean only_visible,
 GSList const *
 gog_plot_get_series (GogPlot const *plot)
 {
-	g_return_val_if_fail (GOG_PLOT (plot) != NULL, NULL);
+	g_return_val_if_fail (IS_GOG_PLOT (plot), NULL);
 	return plot->series;
 }
 
@@ -672,7 +839,7 @@ gog_plot_set_axis_by_id (GogPlot *plot, GogAxisType type, unsigned id)
 	if (id == 0)
 		return FALSE;
 
-	g_return_val_if_fail (GOG_PLOT (plot) != NULL, FALSE);
+	g_return_val_if_fail (IS_GOG_PLOT (plot), FALSE);
 	g_return_val_if_fail (GOG_OBJECT (plot)->parent != NULL, FALSE);
 
 	chart = gog_plot_get_chart (plot);
@@ -736,7 +903,7 @@ gog_plot_axis_clear (GogPlot *plot, GogAxisSet filter)
 {
 	GogAxisType type;
 
-	g_return_if_fail (GOG_PLOT (plot) != NULL);
+	g_return_if_fail (IS_GOG_PLOT (plot));
 
 	for (type = 0 ; type < GOG_AXIS_TYPES ; type++)
 		if (plot->axis[type] != NULL && ((1 << type) & filter)) {
@@ -756,7 +923,7 @@ gog_plot_get_axis_id (GogPlot const *plot, GogAxisType type)
 GogAxis	*
 gog_plot_get_axis (GogPlot const *plot, GogAxisType type)
 {
-	g_return_val_if_fail (GOG_PLOT (plot) != NULL, NULL);
+	g_return_val_if_fail (IS_GOG_PLOT (plot), NULL);
 	g_return_val_if_fail (type < GOG_AXIS_TYPES, NULL);
 	g_return_val_if_fail (GOG_AXIS_UNKNOWN < type, NULL);
 	return plot->axis[type];
@@ -766,7 +933,7 @@ void
 gog_plot_update_3d (GogPlot *plot)
 {
 	GogPlotClass *klass = GOG_PLOT_GET_CLASS (plot);
-	g_return_if_fail (GOG_PLOT (plot) != NULL);
+	g_return_if_fail (IS_GOG_PLOT (plot));
 
 	if (klass->update_3d)
 		klass->update_3d (plot);
@@ -799,7 +966,7 @@ gog_plot_guru_helper (GogPlot *plot)
 	char *hint;
 	unsigned i;
 
- 	g_return_if_fail (GOG_PLOT (plot) != NULL);	
+ 	g_return_if_fail (IS_GOG_PLOT (plot));	
 	klass = GOG_PLOT_GET_CLASS (plot);
 
 	if (plot->guru_hints == NULL)
@@ -824,8 +991,169 @@ gog_plot_guru_helper (GogPlot *plot)
 	g_strfreev (hints);
 }
 
-/****************************************************************************/
-/* a placeholder.  It seems likely that we will want this eventually        */
+/*****************************************************************************/
+
+#ifdef GOFFICE_WITH_GTK
+typedef struct {
+	GogViewAllocation	 start_position;
+	GogViewAllocation	 chart_allocation;
+	GogChart		*chart;
+} MovePlotAreaData;
+
+static gboolean
+gog_tool_move_plot_area_point (GogView *view, double x, double y, GogObject **gobj)
+{
+	GogViewAllocation const *plot_area = gog_chart_view_get_plot_area (view->parent);
+	
+	return (x >= plot_area->x &&
+		x <= (plot_area->x + plot_area->w) &&
+		y >= plot_area->y &&
+		y <= (plot_area->y + plot_area->h));
+}
+
+static void
+gog_tool_move_plot_area_render (GogView *view)
+{
+	GogViewAllocation const *plot_area = gog_chart_view_get_plot_area (view->parent);
+	ArtVpath *path;
+
+	path = gog_renderer_get_rectangle_vpath (plot_area);
+	gog_renderer_draw_sharp_path (view->renderer, path);
+	art_free (path);
+}
+
+static void
+gog_tool_move_plot_area_init (GogToolAction *action)
+{
+	MovePlotAreaData *data = g_new0 (MovePlotAreaData, 1);
+
+	data->chart = GOG_CHART (action->view->parent->model);
+	data->chart_allocation = action->view->parent->allocation;
+	data->start_position = *gog_chart_view_get_plot_area (action->view->parent); 
+	data->start_position.x = (data->start_position.x - data->chart_allocation.x) / data->chart_allocation.w; 
+	data->start_position.w = data->start_position.w / data->chart_allocation.w; 
+	data->start_position.y = (data->start_position.y - data->chart_allocation.y) / data->chart_allocation.h; 
+	data->start_position.h = data->start_position.h / data->chart_allocation.h; 
+
+	action->data = data;
+}
+
+static void
+gog_tool_move_plot_area_move (GogToolAction *action, double x, double y)
+{	
+	GogViewAllocation plot_area;
+	MovePlotAreaData *data = action->data;
+	
+	plot_area.w = data->start_position.w;
+	plot_area.h = data->start_position.h;
+	plot_area.x = data->start_position.x + (x - action->start_x) / data->chart_allocation.w;
+	if (plot_area.x < 0.0)
+		plot_area.x = 0.0;
+	else if (plot_area.x + plot_area.w > 1.0)
+		plot_area.x = 1.0 - plot_area.w;
+	plot_area.y = data->start_position.y + (y - action->start_y) / data->chart_allocation.h;
+	if (plot_area.y < 0.0)
+		plot_area.y = 0.0;
+	else if (plot_area.y + plot_area.h > 1.0)
+		plot_area.y = 1.0 - plot_area.h;
+	gog_chart_set_plot_area (data->chart, &plot_area);
+}
+
+static void
+gog_tool_move_plot_area_double_click (GogToolAction *action)
+{
+	MovePlotAreaData *data = action->data;
+	
+	gog_chart_set_plot_area (data->chart, NULL);
+}
+
+static GogTool gog_tool_move_plot_area = {
+	N_("Move plot area"), 
+	GDK_FLEUR,
+	gog_tool_move_plot_area_point, 
+	gog_tool_move_plot_area_render,
+	gog_tool_move_plot_area_init, 
+	gog_tool_move_plot_area_move, 
+	gog_tool_move_plot_area_double_click,
+	NULL /*destroy*/
+};
+
+static gboolean
+gog_tool_resize_plot_area_point (GogView *view, double x, double y, GogObject **gobj)
+{
+	GogViewAllocation const *plot_area = gog_chart_view_get_plot_area (view->parent);
+
+	return gog_renderer_in_grip (x, y,
+				     plot_area->x + plot_area->w,
+				     plot_area->y + plot_area->h);
+}
+
+static void
+gog_tool_resize_plot_area_render (GogView *view)
+{
+	GogViewAllocation const *plot_area = gog_chart_view_get_plot_area (view->parent);
+
+	gog_renderer_draw_grip (view->renderer, 
+				plot_area->x + plot_area->w,
+				plot_area->y + plot_area->h);
+}
+
+static void
+gog_tool_resize_plot_area_move (GogToolAction *action, double x, double y)
+{	
+	GogViewAllocation plot_area;
+	MovePlotAreaData *data = action->data;
+	
+	plot_area.x = data->start_position.x;
+	plot_area.y = data->start_position.y;
+	plot_area.w = data->start_position.w + (x - action->start_x) / data->chart_allocation.w;
+	if (plot_area.w + plot_area.x > 1.0)
+		plot_area.w = 1.0 - plot_area.x;
+	else if (plot_area.w < 0.0)
+		plot_area.w = 0.0;
+	plot_area.h = data->start_position.h + (y - action->start_y) / data->chart_allocation.h;
+	if (plot_area.h + plot_area.y > 1.0)
+		plot_area.h = 1.0 - plot_area.y;
+	else if (plot_area.h < 0.0)
+		plot_area.h = 0.0;
+	gog_chart_set_plot_area (data->chart, &plot_area);
+}
+
+static GogTool gog_tool_resize_plot_area = {
+	N_("Resize plot area"),
+	GDK_BOTTOM_RIGHT_CORNER,
+	gog_tool_resize_plot_area_point, 
+	gog_tool_resize_plot_area_render,
+	gog_tool_move_plot_area_init, 
+	gog_tool_resize_plot_area_move, 
+	NULL /* double-click */,
+	NULL /*destroy*/
+};
+#endif
+
+/*****************************************************************************/
+
+static GogViewClass *pview_parent_klass;
+
+static void
+gog_plot_view_build_toolkit (GogView *view)
+{
+#ifdef GOFFICE_WITH_GTK
+	view->toolkit = g_slist_prepend (view->toolkit, &gog_tool_move_plot_area);
+	view->toolkit = g_slist_prepend (view->toolkit, &gog_tool_resize_plot_area);
+#endif
+}
+
+static void
+gog_plot_view_class_init (GogPlotViewClass *pview_klass)
+{
+	GogViewClass *view_klass = (GogViewClass *) pview_klass;
+
+	pview_parent_klass = g_type_class_peek_parent (pview_klass);
+
+	view_klass->build_toolkit 	= gog_plot_view_build_toolkit;
+}
+
 GSF_CLASS_ABSTRACT (GogPlotView, gog_plot_view,
-		    NULL, NULL,
+		    gog_plot_view_class_init, NULL,
 		    GOG_VIEW_TYPE)
