@@ -21,13 +21,8 @@
 
 /* TODO:
  *
- * 	- implement stretched image texture
- * 	- fix alpha channel of source image for texture
- * 	- fix font size
- * 	- implement mutiline text
- * 	- cache font properties
- * 	- implement rendering of marker with cairo 
- * 	  (that will fix grayish outline bug)
+ * 	- take screen resolution into account for image and patterns
+ * 	- cache pango_layout
  */
 
 #include <goffice/goffice-config.h>
@@ -46,9 +41,20 @@
 #include <libart_lgpl/art_render_svp.h>
 #include <libart_lgpl/art_render_mask.h>
 #include <pango/pangoft2.h>
-#include <gsf/gsf-impl-utils.h>
 
-#include <cairo/cairo.h>
+#include <gsf/gsf-impl-utils.h>
+#include <gsf/gsf-output-stdio.h>
+
+#include <cairo.h>
+#ifdef CAIRO_HAS_PDF_SURFACE
+#include <cairo-pdf.h>
+#endif
+#ifdef CAIRO_HAS_PS_SURFACE
+#include <cairo-ps.h>
+#endif
+#ifdef CAIRO_HAS_SVG_SURFACE
+#include <cairo-svg.h>
+#endif
 
 #include <math.h>
 
@@ -59,16 +65,21 @@
 
 #define GO_COLOR_TO_CAIRO(x) DOUBLE_RGBA_R(x),DOUBLE_RGBA_G(x),DOUBLE_RGBA_B(x),DOUBLE_RGBA_A(x)
 
+static void grc_free_marker_data (GogRendererCairo *crend);
+
 struct _GogRendererCairo {
 	GogRenderer base;
 
 	int 		 w, h;
 
+	gboolean	 is_vector;
+
 	cairo_t		*cairo;
 	GdkPixbuf 	*pixbuf;
 
 	cairo_surface_t *marker_surface;
-	GdkPixbuf	*marker_pixbuf;
+	double		 marker_x_offset;
+	double		 marker_y_offset;
 };
 
 typedef GogRendererClass GogRendererCairoClass;
@@ -79,11 +90,8 @@ static void
 gog_renderer_cairo_finalize (GObject *obj)
 {
 	GogRendererCairo *crend = GOG_RENDERER_CAIRO (obj);
-	cairo_surface_t *surface;
 
 	if (crend->cairo != NULL){
-		surface = cairo_get_target (crend->cairo);
-		cairo_surface_destroy (surface);
 		cairo_destroy (crend->cairo);
 		crend->cairo = NULL;
 	}
@@ -91,44 +99,30 @@ gog_renderer_cairo_finalize (GObject *obj)
 		g_object_unref (crend->pixbuf);
 		crend->pixbuf = NULL;
 	}
-	if (crend->marker_surface != NULL) {
-		cairo_surface_destroy (crend->marker_surface);
-		g_object_unref (crend->marker_pixbuf);
-		crend->marker_surface = NULL;
-		crend->marker_pixbuf = NULL;
-	}
+	grc_free_marker_data (crend);
 	
 	(*parent_klass->finalize) (obj);
 }
 
 static void
-gog_renderer_cairo_clip_push (GogRenderer *rend, GogRendererClip *clip)
+grc_free_marker_data (GogRendererCairo *crend)
 {
-	GogRendererCairo *crend = GOG_RENDERER_CAIRO (rend);
-	double x, y;
-
-	/* Rounding clipping area trigger cairo fast clip */
-	x = floor (clip->area.x + 0.5);
-	y = floor (clip->area.y + 0.5);
-
-	cairo_save (crend->cairo);
-	cairo_rectangle (crend->cairo, x, y,
-			 floor (clip->area.x + clip->area.w + 0.5) - x,
-			 floor (clip->area.y + clip->area.h + 0.5) - y);
-	cairo_clip (crend->cairo);
-}
-
-static void
-gog_renderer_cairo_clip_pop (GogRenderer *rend, GogRendererClip *clip)
-{
-	GogRendererCairo *crend = GOG_RENDERER_CAIRO (rend);
-
-	cairo_restore (crend->cairo);
+	if (crend->marker_surface != NULL) {
+		cairo_surface_destroy (crend->marker_surface);
+		crend->marker_surface = NULL;
+	}
 }
 
 static double
 grc_line_size (GogRenderer const *rend, double width)
 {
+	GogRendererCairo *crend = GOG_RENDERER_CAIRO (rend);
+
+	if (crend->is_vector)
+		return go_sub_epsilon (width) <= 0. ?
+			GOG_RENDERER_HAIR_LINE_WIDTH :
+			width * rend->scale;
+
 	if (go_sub_epsilon (width) <= 0.) /* cheesy version of hairline */
 		return 1.;
 	
@@ -139,36 +133,6 @@ grc_line_size (GogRenderer const *rend, double width)
 	return floor (width);
 }
 
-static double
-gog_renderer_cairo_line_size (GogRenderer const *rend, double width)
-{
-	double size = grc_line_size (rend, width);
-
-	if (size < 1.0)
-		return ceil (size);
-
-	return size;
-}
-
-static void
-gog_renderer_cairo_sharp_path (GogRenderer *rend, ArtVpath *path, double line_width) 
-{
-	ArtVpath *iter = path;
-
-	if (((int) (rint (line_width)) % 2 == 0) && line_width > 1.0) 
-		while (iter->code != ART_END) {
-			iter->x = floor (iter->x + .5);
-			iter->y = floor (iter->y + .5);
-			iter++;
-		}
-	else
-		while (iter->code != ART_END) {
-			iter->x = floor (iter->x) + .5;
-			iter->y = floor (iter->y) + .5;
-			iter++;
-		}
-}
-
 static void
 grc_path (cairo_t *cr, ArtVpath *vpath, ArtBpath *bpath)
 {
@@ -212,20 +176,33 @@ grc_path (cairo_t *cr, ArtVpath *vpath, ArtBpath *bpath)
 
 /* Red and blue are inverted in a pixbuf compared to cairo */
 static void
-grc_invert_pixbuf_RB (unsigned char *pixels, int width, int height, int rowstride)
+grc_pixbuf_to_cairo (unsigned char *p, int width, int height, int rowstride)
 {
 	int i,j;
 	unsigned char a;
+	guint t;
+	
+#define MULT(d,c,a,t) G_STMT_START { t = c * a + 0x7f; d = ((t >> 8) + t) >> 8; } G_STMT_END
 	
 	for (i = 0; i < height; i++) {
 		for (j = 0; j < width; j++) {
-			a = pixels[0];
-			pixels[0] = pixels[2];
-			pixels[2] = a;
-			pixels += 4;
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+			MULT(a,    p[2], p[3], t);
+			MULT(p[1], p[1], p[3], t);
+			MULT(p[2], p[0], p[3], t);
+			p[0] = a;
+#else	  
+			a = p[3];
+			MULT(p[3], p[2], a, t);
+			MULT(p[2], p[1], a, t);
+			MULT(p[1], p[0], a, t);
+			p[0] = a;
+#endif
+			p += 4;
 		}
-		pixels += rowstride - width * 4;
+		p += rowstride - width * 4;
 	}
+#undef MULT
 }
 
 static void
@@ -238,17 +215,102 @@ grc_draw_path (GogRenderer *rend, ArtVpath const *vpath, ArtBpath const*bpath)
 
 	g_return_if_fail (bpath != NULL || vpath != NULL);
 
+	if (style->line.dash_type == GO_LINE_NONE)
+		return;
+
 	cairo_set_line_width (cr, width);
 	if (rend->line_dash != NULL)
 		cairo_set_dash (cr, 
 				rend->line_dash->dash, 
 				rend->line_dash->n_dash, 
 				rend->line_dash->offset);
+	else
+		cairo_set_dash (cr, NULL, 0, 0);
 	grc_path (cr, (ArtVpath *) vpath, (ArtBpath *) bpath);
 	cairo_set_source_rgba (cr, GO_COLOR_TO_CAIRO (style->line.color));
 	cairo_stroke (cr);
 }
 
+static void
+gog_renderer_cairo_push_clip (GogRenderer *rend, GogRendererClip *clip)
+{
+	GogRendererCairo *crend = GOG_RENDERER_CAIRO (rend);
+	ArtVpath *path = clip->path;
+	int i;
+	gboolean is_rectangle;
+
+	if (!crend->is_vector) {
+		for (i = 0; i < 6; i++)
+			if (path[i].code == ART_END)
+				break;
+
+		is_rectangle = i == 5 &&
+			path[5].code == ART_END &&
+			path[0].x == path[3].x &&
+			path[0].x == path[4].x &&
+			path[1].x == path[2].x &&
+			path[0].y == path[1].y &&
+			path[0].y == path[4].y &&
+			path[2].y == path[3].y;
+	} else 
+		is_rectangle = FALSE;
+
+	cairo_save (crend->cairo);
+	if (is_rectangle) {
+		double x = floor (path[0].x + 0.5);
+		double y = floor (path[0].y + 0.5);
+		cairo_rectangle (crend->cairo, x, y,
+				 floor (path[1].x + 0.5) - x,
+				 floor (path[2].y + 0.5) - y);
+	} else {
+		grc_path (crend->cairo, path, NULL);
+	}
+	cairo_clip (crend->cairo);
+}
+
+static void
+gog_renderer_cairo_pop_clip (GogRenderer *rend, GogRendererClip *clip)
+{
+	GogRendererCairo *crend = GOG_RENDERER_CAIRO (rend);
+
+	cairo_restore (crend->cairo);
+}
+
+static double
+gog_renderer_cairo_line_size (GogRenderer const *rend, double width)
+{
+	GogRendererCairo *crend = GOG_RENDERER_CAIRO (rend);
+	double size = grc_line_size (rend, width);
+
+	if (!crend->is_vector && size < 1.0)
+		return ceil (size);
+
+	return size;
+}
+
+static void
+gog_renderer_cairo_sharp_path (GogRenderer *rend, ArtVpath *path, double line_width) 
+{
+	GogRendererCairo *crend = GOG_RENDERER_CAIRO (rend);
+	ArtVpath *iter = path;
+
+	if (crend->is_vector)
+		return;
+
+	if (((int) (rint (line_width)) % 2 == 0) && line_width > 1.0) 
+		while (iter->code != ART_END) {
+			iter->x = floor (iter->x + .5);
+			iter->y = floor (iter->y + .5);
+			iter++;
+		}
+	else
+		while (iter->code != ART_END) {
+			iter->x = floor (iter->x) + .5;
+			iter->y = floor (iter->y) + .5;
+			iter++;
+		}
+}
+
 static void
 gog_renderer_cairo_draw_path (GogRenderer *rend, ArtVpath const *path)
 {
@@ -289,12 +351,13 @@ grc_draw_polygon (GogRenderer *rend, ArtVpath const *vpath,
 	cairo_t *cr = crend->cairo;
 	cairo_pattern_t *cr_pattern = NULL;
 	cairo_surface_t *cr_surface = NULL;
+	cairo_matrix_t cr_matrix;
 	GdkPixbuf *pixbuf = NULL;
 	GOColor color;
-	double width = grc_line_size (rend, style->line.width);
+	double width = grc_line_size (rend, style->outline.width);
 	double x[3], y[3];
 	int i, j, w, h, rowstride;
-	char const *pattern;
+	guint8 const *pattern;
 	unsigned char *pixels, *iter;
 
 	g_return_if_fail (bpath != NULL || vpath != NULL);
@@ -361,6 +424,7 @@ grc_draw_polygon (GogRenderer *rend, ArtVpath const *vpath,
 				cairo_set_source_rgba (cr, 1, 1, 1, 1); 
 				break;
 			}
+			cairo_fill_extents (cr, &x[0], &y[0], &x[1], &y[1]);
 			pixbuf = gdk_pixbuf_add_alpha (style->fill.image.image, FALSE, 0, 0, 0);
 			pixels = gdk_pixbuf_get_pixels (pixbuf);
 			h = gdk_pixbuf_get_height (pixbuf);
@@ -368,9 +432,26 @@ grc_draw_polygon (GogRenderer *rend, ArtVpath const *vpath,
 			rowstride = gdk_pixbuf_get_rowstride (pixbuf);
 			cr_surface = cairo_image_surface_create_for_data (pixels,
 				CAIRO_FORMAT_ARGB32, w, h, rowstride);
-			grc_invert_pixbuf_RB (pixels, w, h, rowstride);
+			grc_pixbuf_to_cairo (pixels, w, h, rowstride);
 			cr_pattern = cairo_pattern_create_for_surface (cr_surface);
 			cairo_pattern_set_extend (cr_pattern, CAIRO_EXTEND_REPEAT);
+			switch (style->fill.image.type) {
+				case GOG_IMAGE_CENTERED:
+					cairo_matrix_init_translate (&cr_matrix, 
+								     -(x[1] - x[0] - w) / 2 - x[0], 
+								     -(y[1] - y[0] - h) / 2 - y[0]);
+					cairo_pattern_set_matrix (cr_pattern, &cr_matrix);
+					break;
+				case GOG_IMAGE_STRETCHED:
+					cairo_matrix_init_scale (&cr_matrix, 
+								 w / (x[1] - x[0]), 
+								 h / (y[1] - y[0]));
+					cairo_matrix_translate (&cr_matrix, -x[0], -y[0]);
+					cairo_pattern_set_matrix (cr_pattern, &cr_matrix);
+					break;
+				case GOG_IMAGE_WALLPAPER:
+					break;
+			}
 			cairo_set_source (cr, cr_pattern);
 			break;
 
@@ -392,6 +473,8 @@ grc_draw_polygon (GogRenderer *rend, ArtVpath const *vpath,
 					rend->outline_dash->dash, 
 					rend->outline_dash->n_dash, 
 					rend->outline_dash->offset);
+		else
+			cairo_set_dash (cr, NULL, 0, 0);
 		cairo_stroke (cr);
 	}
 
@@ -422,39 +505,25 @@ gog_renderer_cairo_draw_text (GogRenderer *rend, char const *text,
 			      GogViewAllocation const *pos, GtkAnchorType anchor,
 			      GogViewAllocation *result)
 {
-	GogRendererCairo *crend = (GogRendererCairo *) rend;
+	GogRendererCairo *crend = GOG_RENDERER_CAIRO (rend);
 	GogStyle const *style = rend->cur_style;
-	PangoFontDescription const *fd = style->font.font->desc;
-	PangoWeight weight;
-	cairo_t *cr = crend->cairo;
-	cairo_text_extents_t text_extents;
-	cairo_font_extents_t font_extents;
-	cairo_font_slant_t slant;
+	PangoLayout *layout;
+	PangoContext *context;
+	cairo_t *cairo = crend->cairo;
 	GOGeometryOBR obr;
 	GOGeometryAABR aabr;
-	char const *family;
-	double size;
+	int iw, ih;
 
-	family = pango_font_description_get_family (fd);
-	size = pango_font_description_get_size (fd) / PANGO_SCALE;
-	weight = pango_font_description_get_weight (fd);
-	switch (pango_font_description_get_style (fd)) {
-		case (PANGO_STYLE_NORMAL):  slant = CAIRO_FONT_SLANT_NORMAL; break;
-		case (PANGO_STYLE_OBLIQUE): slant = CAIRO_FONT_SLANT_OBLIQUE; break;
-		case (PANGO_STYLE_ITALIC):  slant = CAIRO_FONT_SLANT_ITALIC; break;
-	}
-/*	g_message ("family: %s, size: %g", family, size);*/
-	/* FIXME: calculate dpi */
-	size *= 96.0 * rend->scale * rend->zoom / 72.0;
-	cairo_select_font_face (cr, family, slant, 
-		weight > PANGO_WEIGHT_SEMIBOLD ?  CAIRO_FONT_WEIGHT_BOLD : CAIRO_FONT_WEIGHT_NORMAL);
-	cairo_set_font_size (cr, size);
-	cairo_text_extents (cr, text, &text_extents);
-	cairo_font_extents (cr, &font_extents);
-	
-	obr.w = text_extents.width;
-	obr.h = font_extents.ascent + font_extents.descent;
-	obr.alpha = rend->cur_style->text_layout.angle * M_PI / 180.0;
+	layout = pango_cairo_create_layout (cairo);
+	context = pango_layout_get_context (layout);
+	pango_cairo_context_set_resolution (context, 72 * rend->scale);
+	pango_layout_set_markup (layout, text, -1);
+	pango_layout_set_font_description (layout, style->font.font->desc);
+	pango_layout_get_size (layout, &iw, &ih);
+
+	obr.w = iw / (double)PANGO_SCALE;
+	obr.h = ih / (double)PANGO_SCALE;
+	obr.alpha = style->text_layout.angle * M_PI / 180.0;
 	obr.x = pos->x;
 	obr.y = pos->y;
 	go_geometry_OBR_to_AABR (&obr, &aabr);
@@ -468,8 +537,6 @@ gog_renderer_cairo_draw_text (GogRenderer *rend, char const *text,
 			break;
 		default : break;
 	}
-	if (obr.x <= 0)
-		obr.x = 0;
 
 	switch (anchor) {
 		case GTK_ANCHOR_NW: case GTK_ANCHOR_N: case GTK_ANCHOR_NE:
@@ -480,16 +547,15 @@ gog_renderer_cairo_draw_text (GogRenderer *rend, char const *text,
 			break;
 		default : break;
 	}
-	if (obr.y <= 0)
-		obr.y = 0;
 
-	cairo_save (cr);
-	cairo_set_source_rgba (cr, GO_COLOR_TO_CAIRO (style->font.color)); 
-	cairo_move_to (cr, obr.x, obr.y);
-	cairo_rotate (cr, -obr.alpha);
-	cairo_rel_move_to (cr, - obr.w / 2.0, + obr.h / 2.0 - font_extents.descent);
-	cairo_show_text (cr, text);
-	cairo_restore (cr);
+	cairo_save (cairo);
+	cairo_set_source_rgba (cairo, GO_COLOR_TO_CAIRO (style->font.color)); 
+	cairo_move_to (cairo, obr.x - (obr.w / 2.0) * cos (obr.alpha) - (obr.h / 2.0) * sin (obr.alpha),
+		       obr.y + (obr.w / 2.0) * sin (obr.alpha) - (obr.h / 2.0) * cos (obr.alpha));
+	cairo_rotate (cairo, -obr.alpha);
+	pango_cairo_show_layout (cairo, layout);
+	cairo_restore (cairo);
+	g_object_unref (layout);
 
 	if (result != NULL) {
 		result->x = aabr.x;
@@ -503,64 +569,84 @@ static void
 gog_renderer_cairo_get_text_OBR (GogRenderer *rend,
 				 char const *text, GOGeometryOBR *obr)
 {
-	GogRendererCairo *crend = (GogRendererCairo *) rend;
+	GogRendererCairo *crend = GOG_RENDERER_CAIRO (rend);
 	GogStyle const *style = rend->cur_style;
-	PangoFontDescription const *fd = style->font.font->desc;
-	PangoWeight weight;
-	cairo_t *cr = crend->cairo;
-	cairo_text_extents_t text_extents;
-	cairo_font_extents_t font_extents;
-	cairo_font_slant_t slant;
-	char const *family;
-	double size;
-	
-	family = pango_font_description_get_family (fd);
-	/* FIXME: calculate dpi */
-	size = pango_font_description_get_size (fd) / PANGO_SCALE / 72.0 * 96.0 * rend->scale * rend->zoom;
-	weight = pango_font_description_get_weight (fd);
-	switch (pango_font_description_get_style (fd)) {
-		case (PANGO_STYLE_NORMAL):  slant = CAIRO_FONT_SLANT_NORMAL; break;
-		case (PANGO_STYLE_OBLIQUE): slant = CAIRO_FONT_SLANT_OBLIQUE; break;
-		case (PANGO_STYLE_ITALIC):  slant = CAIRO_FONT_SLANT_ITALIC; break;
-	}
-	cairo_select_font_face (cr, family, slant, 
-		weight > PANGO_WEIGHT_SEMIBOLD ?  CAIRO_FONT_WEIGHT_BOLD : CAIRO_FONT_WEIGHT_NORMAL);
-	cairo_set_font_size (cr, size);
-	cairo_text_extents (cr, text, &text_extents);
-	cairo_font_extents (cr, &font_extents);
+	PangoLayout *layout;
+	PangoContext *context;
+	PangoRectangle logical;
+	cairo_t *cairo = crend->cairo;
 
-	obr->w = text_extents.width;
-	obr->h = font_extents.ascent + font_extents.descent;
+	layout = pango_cairo_create_layout (cairo);
+	context = pango_layout_get_context (layout);
+	pango_cairo_context_set_resolution (context, 72 * rend->scale);
+	pango_layout_set_markup (layout, text, -1);
+	pango_layout_set_font_description (layout, style->font.font->desc);
+	pango_layout_get_extents (layout, NULL, &logical);
+	g_object_unref (layout);
+	
+	obr->w = ((double) logical.width + (double) PANGO_SCALE / 2.0) / (double) PANGO_SCALE;
+	obr->h = ((double) logical.height + (double) PANGO_SCALE / 2.0) /(double) PANGO_SCALE;
 }
 
 static cairo_surface_t *
 grc_get_marker_surface (GogRenderer *rend)
 {
 	GogRendererCairo *crend = GOG_RENDERER_CAIRO (rend);
-	GogStyle const *style = rend->cur_style;
-	GdkPixbuf *pixbuf, *marker_pixbuf;
+	GOMarker *marker = rend->cur_style->marker.mark;
+	cairo_t *cairo;
 	cairo_surface_t *surface;
-	unsigned char *pixels;
-	int height, width, rowstride;
+	ArtVpath *path;
+	ArtVpath const *outline_path_raw, *fill_path_raw;
+	double scaling[6], translation[6], affine[6];
+	double half_size, offset;
 
 	if (crend->marker_surface != NULL)
 		return crend->marker_surface;
-       	
-	marker_pixbuf = go_marker_get_pixbuf (style->marker.mark, rend->scale);
-	if (marker_pixbuf == NULL)
+
+	go_marker_get_paths (marker, &outline_path_raw, &fill_path_raw);
+
+	if ((outline_path_raw == NULL) ||
+	    (fill_path_raw == NULL))
 		return NULL;
 
-	pixbuf = gdk_pixbuf_copy (go_marker_get_pixbuf (style->marker.mark, rend->scale));
-       	pixels = gdk_pixbuf_get_pixels (pixbuf);
-	height = gdk_pixbuf_get_height (pixbuf);
-	width = gdk_pixbuf_get_width (pixbuf);
-	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-	surface = cairo_image_surface_create_for_data (pixels,
-		CAIRO_FORMAT_ARGB32, width, height, rowstride);
-	grc_invert_pixbuf_RB (pixels, width, height, rowstride);
+	if (crend->is_vector) {
+		half_size = rend->scale * go_marker_get_size (marker) / 2.0;
+		offset = half_size + rend->scale * go_marker_get_outline_width (marker);
+	} else {
+		half_size = rint (rend->scale * go_marker_get_size (marker)) / 2.0;
+		offset = ceil (rend->scale * go_marker_get_outline_width (marker) / 2.0) + 
+			half_size + .5;
+	}
+	surface = cairo_surface_create_similar (cairo_get_target (crend->cairo),
+						CAIRO_CONTENT_COLOR_ALPHA,
+						2.0 * offset, 
+						2.0 * offset);
+	cairo = cairo_create (surface);
+
+	cairo_set_line_cap (cairo, CAIRO_LINE_CAP_SQUARE);
+	cairo_set_line_join (cairo, CAIRO_LINE_JOIN_MITER);
+						       
+	art_affine_scale (scaling, half_size, half_size);
+	art_affine_translate (translation, offset, offset);
+	art_affine_multiply (affine, scaling, translation);
+
+	path = art_vpath_affine_transform (fill_path_raw, affine);
+	cairo_set_source_rgba (cairo, GO_COLOR_TO_CAIRO (go_marker_get_fill_color (marker)));
+	grc_path (cairo, path, NULL);
+	cairo_fill (cairo);
+	art_free (path);
+
+	path = art_vpath_affine_transform (outline_path_raw, affine);
+	cairo_set_source_rgba (cairo, GO_COLOR_TO_CAIRO (go_marker_get_outline_color (marker)));
+	cairo_set_line_width (cairo, rend->scale * go_marker_get_outline_width (marker));
+	grc_path (cairo, path, NULL);
+	cairo_stroke (cairo);
+	art_free (path);
+
+	cairo_destroy (cairo);
 
-	crend->marker_pixbuf = pixbuf;
 	crend->marker_surface = surface;
+	crend->marker_x_offset = crend->marker_y_offset = offset;
 
 	return surface;
 }
@@ -570,22 +656,29 @@ gog_renderer_cairo_draw_marker (GogRenderer *rend, double x, double y)
 {
 	GogRendererCairo *crend = GOG_RENDERER_CAIRO (rend);
 	cairo_surface_t *surface;
-	double width, height;
 
 	surface = grc_get_marker_surface (rend);
+
 	if (surface == NULL)
 		return;
-	width = cairo_image_surface_get_width (surface);
-	height = cairo_image_surface_get_height (surface);
-	cairo_set_source_surface (crend->cairo, surface, 
-				  floor (floor (x + .5) - width / 2.0),
-				  floor (floor (y + .5) - height / 2.0));
+
+	if (crend->is_vector)
+		cairo_set_source_surface (crend->cairo, surface, 
+					  x - crend->marker_x_offset, 
+					  y - crend->marker_y_offset);
+	else 
+		cairo_set_source_surface (crend->cairo, surface, 
+					  floor (x - crend->marker_x_offset), 
+					  floor (y - crend->marker_y_offset));
 	cairo_paint (crend->cairo);
 }
 
 static void
 gog_renderer_cairo_push_style (GogRenderer *rend, GogStyle const *style)
 {
+	GogRendererCairo *crend = GOG_RENDERER_CAIRO (rend);
+	
+	grc_free_marker_data (crend);
 }
 
 static void
@@ -593,11 +686,130 @@ gog_renderer_cairo_pop_style (GogRenderer *rend)
 {
 	GogRendererCairo *crend = GOG_RENDERER_CAIRO (rend);
 	
-	if (crend->marker_surface != NULL) {
-		cairo_surface_destroy (crend->marker_surface);
-		g_object_unref (crend->marker_pixbuf);
-		crend->marker_surface = NULL;
-		crend->marker_pixbuf = NULL;
+	grc_free_marker_data (crend);
+}
+
+static gboolean
+grc_gsf_gdk_pixbuf_save (const gchar *buf,
+			 gsize count,
+			 GError **error,
+			 gpointer data)
+{
+	GsfOutput *output = GSF_OUTPUT (data);
+	gboolean ok = gsf_output_write (output, count, buf);
+
+	if (!ok && error)
+		*error = g_error_copy (gsf_output_error (output));
+
+	return ok;
+}
+
+static cairo_status_t
+grc_cairo_write_func (void *closure,
+		  const unsigned char *data,
+		  unsigned int length)
+{
+	gboolean result;
+	GsfOutput *output = GSF_OUTPUT (closure);
+
+	result = gsf_output_write (output, length, data);
+
+	return result ? CAIRO_STATUS_SUCCESS : CAIRO_STATUS_WRITE_ERROR;
+}
+
+static gboolean
+gog_renderer_cairo_export_image (GogRenderer *renderer, GOImageFormat format,
+				 GsfOutput *output, double x_dpi, double y_dpi)
+{
+	GogRendererCairo *crend = GOG_RENDERER_CAIRO (renderer);
+	GogViewAllocation allocation;
+	GOImageFormatInfo const *format_info;
+	cairo_surface_t *surface = NULL;
+	cairo_status_t status;
+	GdkPixbuf *pixbuf;
+	double width_in_pts, height_in_pts;
+
+	gog_graph_get_size (renderer->model, &width_in_pts, &height_in_pts);
+
+	switch (format) {
+		case GO_IMAGE_FORMAT_PNG:
+		case GO_IMAGE_FORMAT_JPG:
+			gog_renderer_cairo_update (crend, 
+						   width_in_pts * x_dpi / 72.0, 
+						   height_in_pts * y_dpi / 72.0, 1.0);
+			pixbuf = gog_renderer_cairo_get_pixbuf (crend);
+			if (pixbuf == NULL)
+				return FALSE;
+			format_info = go_image_get_format_info (format);
+			return gdk_pixbuf_save_to_callback (pixbuf,
+							    grc_gsf_gdk_pixbuf_save,
+							    output, format_info->name,
+							    NULL, NULL);
+			break;
+		case GO_IMAGE_FORMAT_PDF:
+		case GO_IMAGE_FORMAT_PS:
+		case GO_IMAGE_FORMAT_SVG:
+			crend->base.scale = 1.0;
+			switch (format) {
+				case GO_IMAGE_FORMAT_PDF:
+#ifdef CAIRO_HAS_PDF_SURFACE
+					surface = cairo_pdf_surface_create_for_stream 
+						(grc_cairo_write_func, 
+						 output, width_in_pts, height_in_pts);
+					break;
+#else
+					g_warning ("[GogRendererCairo::export_image] cairo PDF backend missing");
+					return;
+#endif
+				case GO_IMAGE_FORMAT_PS:
+#ifdef CAIRO_HAS_PS_SURFACE
+					surface = cairo_ps_surface_create_for_stream 
+						(grc_cairo_write_func, 
+						 output, width_in_pts, height_in_pts);
+					break;
+#else
+					g_warning ("[GogRendererCairo::export_image] cairo PS backend missing");
+					return;
+#endif
+				case GO_IMAGE_FORMAT_SVG:
+#ifdef CAIRO_HAS_SVG_SURFACE
+					surface = cairo_svg_surface_create_for_stream 
+						(grc_cairo_write_func, 
+						 output, width_in_pts, height_in_pts);
+					break;
+#else
+					g_warning ("[GogRendererCairo::export_image] cairo SVG backend missing");
+					return;
+#endif
+				default:
+					break;
+			}
+			crend->cairo = cairo_create (surface);
+			cairo_surface_destroy (surface);
+			cairo_set_line_join (crend->cairo, CAIRO_LINE_JOIN_ROUND);
+			cairo_set_line_cap (crend->cairo, CAIRO_LINE_CAP_ROUND);
+			crend->w = width_in_pts;
+			crend->h = height_in_pts;
+			crend->is_vector = TRUE;
+
+			allocation.x = 0.;
+			allocation.y = 0.;
+			allocation.w = width_in_pts;
+			allocation.h = height_in_pts;
+			gog_view_size_allocate (crend->base.view, &allocation);
+			gog_view_render	(crend->base.view, NULL);
+
+			cairo_show_page (crend->cairo);
+			status = cairo_status (crend->cairo);
+
+			cairo_destroy (crend->cairo);
+			crend->cairo = NULL;
+
+			return status == CAIRO_STATUS_SUCCESS;
+			break;
+		default:
+			g_warning ("[GogRendererCairo:export_image] unsupported format");
+			return FALSE;
 	}
 }
 
@@ -610,8 +822,8 @@ gog_renderer_cairo_class_init (GogRendererClass *rend_klass)
 	gobject_klass->finalize		= gog_renderer_cairo_finalize;
 	rend_klass->push_style		= gog_renderer_cairo_push_style;
 	rend_klass->pop_style		= gog_renderer_cairo_pop_style;
-	rend_klass->clip_push  		= gog_renderer_cairo_clip_push;
-	rend_klass->clip_pop     	= gog_renderer_cairo_clip_pop;
+	rend_klass->push_clip  		= gog_renderer_cairo_push_clip;
+	rend_klass->pop_clip     	= gog_renderer_cairo_pop_clip;
 	rend_klass->sharp_path		= gog_renderer_cairo_sharp_path;
 	rend_klass->draw_path	  	= gog_renderer_cairo_draw_path;
 	rend_klass->draw_polygon  	= gog_renderer_cairo_draw_polygon;
@@ -621,6 +833,7 @@ gog_renderer_cairo_class_init (GogRendererClass *rend_klass)
 	rend_klass->draw_marker	  	= gog_renderer_cairo_draw_marker;
 	rend_klass->get_text_OBR	= gog_renderer_cairo_get_text_OBR;
 	rend_klass->line_size		= gog_renderer_cairo_line_size;
+	rend_klass->export_image	= gog_renderer_cairo_export_image;
 }
 
 static void
@@ -629,68 +842,20 @@ gog_renderer_cairo_init (GogRendererCairo *crend)
 	crend->cairo = NULL;
 	crend->pixbuf = NULL;
 	crend->marker_surface = NULL;
-	crend->marker_pixbuf = NULL;
 	crend->w = crend->h = 0;
+	crend->is_vector = FALSE;
 }
 
 GSF_CLASS (GogRendererCairo, gog_renderer_cairo,
 	   gog_renderer_cairo_class_init, gog_renderer_cairo_init,
 	   GOG_RENDERER_TYPE)
 
-GdkPixbuf *
-gog_renderer_cairo_get_pixbuf (GogRendererCairo *crend)
-{
-	g_return_val_if_fail (crend != NULL, NULL);
-
-	return crend->pixbuf;
-}
-
-static gboolean
-grc_cairo_setup (GogRendererCairo *crend, int w, int h)
-{
-	cairo_surface_t *surface;
-
-	if (w == crend->w && h == crend->h)
-		return (w != 0 && h!= 0);
-		
-	if (crend->cairo != NULL) {
-		surface = cairo_get_target (crend->cairo);
-		cairo_surface_destroy (surface);
-		cairo_destroy (crend->cairo);
-		crend->cairo = NULL;
-	}
-	if (crend->pixbuf != NULL) {
-		g_object_unref (crend->pixbuf);
-		crend->pixbuf = NULL;
-	}
-	crend->w = w;
-	crend->h = h;
-
-	if (w ==0 || h == 0) 
-		return FALSE;
-	
-	crend->pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, crend->w, crend->h);
-	if (crend->pixbuf == NULL) {
-		g_warning ("GogRendererCairo::cairo_setup: chart is too large");
-		return FALSE;
-	}
-
-	surface = cairo_image_surface_create_for_data (gdk_pixbuf_get_pixels (crend->pixbuf),
-						       CAIRO_FORMAT_ARGB32, 
-						       crend->w, crend->h,
-						       gdk_pixbuf_get_rowstride (crend->pixbuf));
-	crend->cairo = cairo_create (surface);
-	cairo_set_line_join (crend->cairo, CAIRO_LINE_JOIN_ROUND);
-	cairo_set_line_cap (crend->cairo, CAIRO_LINE_CAP_ROUND);
-
-	return TRUE;
-}
-
 /**
- * gog_renderer_update :
- * @prend :
- * @w :
- * @h :
+ * gog_renderer_cairo_update:
+ * @crend: a #GogRendererCairo
+ * @w:
+ * @h:
+ * @zoom:
  *
  * Returns TRUE if the size actually changed.
  **/
@@ -700,22 +865,48 @@ gog_renderer_cairo_update (GogRendererCairo *crend, int w, int h, double zoom)
 	GogGraph *graph;
 	GogView *view;
 	GogViewAllocation allocation;
+	cairo_surface_t *surface;
 	gboolean redraw = TRUE;
 	gboolean size_changed;
 
-	g_return_val_if_fail (crend != NULL, FALSE);
-	g_return_val_if_fail (crend->base.view != NULL, FALSE);
+	g_return_val_if_fail (IS_GOG_RENDERER_CAIRO (crend), FALSE);
+	g_return_val_if_fail (IS_GOG_VIEW (crend->base.view), FALSE);
 
 	size_changed = crend->w != w || crend->h != h;
+	if (size_changed) {
+		if (crend->pixbuf != NULL)
+			g_object_unref (crend->pixbuf);
+		crend->pixbuf = NULL;
+		if (w == 0 || h == 0)
+			return FALSE;
+		crend->w = w;
+		crend->h = h;
+		crend->pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, crend->w, crend->h);
+		if (crend->pixbuf == NULL) {
+			crend->w = 0;
+			crend->h = 0;
+			g_warning ("GogRendererCairo::cairo_setup: chart is too large");
+			return FALSE;
+		}
+	}
+	if (w == 0 || h == 0)
+		return FALSE;
 
 	view = crend->base.view;
 	graph = GOG_GRAPH (view->model);
 	gog_graph_force_update (graph);
+	
 	allocation.x = allocation.y = 0.;
 	allocation.w = w;
 	allocation.h = h;
-	if (!grc_cairo_setup (crend, w, h))
-		return redraw;
+	
+	surface = cairo_image_surface_create_for_data (gdk_pixbuf_get_pixels (crend->pixbuf),
+						       CAIRO_FORMAT_ARGB32, 
+						       crend->w, crend->h,
+						       gdk_pixbuf_get_rowstride (crend->pixbuf));
+	crend->cairo = cairo_create (surface);
+	cairo_surface_destroy (surface);
+	crend->is_vector = FALSE;
 
 	if (size_changed) {
 		crend->base.scale_x = w / graph->width;
@@ -739,15 +930,29 @@ gog_renderer_cairo_update (GogRendererCairo *crend, int w, int h, double zoom)
 	crend->base.needs_update = FALSE;
 
 	if (redraw) { 
-		cairo_rectangle (crend->cairo, 0, 0, w, h);
-		cairo_set_source_rgba (crend->cairo, 1, 1, 1, 0);
-		cairo_fill (crend->cairo);
-		
+		cairo_set_operator (crend->cairo, CAIRO_OPERATOR_CLEAR);
+		cairo_paint (crend->cairo);
+		cairo_set_operator (crend->cairo, CAIRO_OPERATOR_OVER);
+
+		cairo_set_line_join (crend->cairo, CAIRO_LINE_JOIN_ROUND);
+		cairo_set_line_cap (crend->cairo, CAIRO_LINE_CAP_ROUND);
+
 		gog_view_render	(view, NULL);
 		
-		grc_invert_pixbuf_RB (gdk_pixbuf_get_pixels (crend->pixbuf), w, h, 
-				      gdk_pixbuf_get_rowstride (crend->pixbuf));
+		grc_pixbuf_to_cairo (gdk_pixbuf_get_pixels (crend->pixbuf), w, h, 
+				     gdk_pixbuf_get_rowstride (crend->pixbuf));
 	}
 
+	cairo_destroy (crend->cairo);
+	crend->cairo = NULL;
 	return redraw;
 }
+
+GdkPixbuf *
+gog_renderer_cairo_get_pixbuf (GogRendererCairo *crend)
+{
+	g_return_val_if_fail (crend != NULL, NULL);
+
+	return crend->pixbuf;
+}
+
