@@ -36,6 +36,7 @@
 #include <libart_lgpl/art_render_mask.h>
 #include <pango/pangoft2.h>
 #include <gsf/gsf-impl-utils.h>
+#include <gsf/gsf-output-stdio.h>
 
 #include <math.h>
 
@@ -943,6 +944,53 @@ gog_renderer_pixbuf_push_style (GogRenderer *rend, GogStyle const *style)
 	}
 }
 
+static gboolean
+grp_gsf_gdk_pixbuf_save (const gchar *buf,
+			 gsize count,
+			 GError **error,
+			 gpointer data)
+{
+	GsfOutput *output = GSF_OUTPUT (data);
+	gboolean ok = gsf_output_write (output, count, buf);
+
+	if (!ok && error)
+		*error = g_error_copy (gsf_output_error (output));
+
+	return ok;
+}
+
+static gboolean
+gog_renderer_pixbuf_export_image (GogRenderer *renderer, GOImageFormat format,
+				  GsfOutput *output, double x_dpi, double y_dpi)
+{
+	GogRendererPixbuf *prend = GOG_RENDERER_PIXBUF (renderer);
+	GdkPixbuf *pixbuf;
+	GOImageFormatInfo const *format_info;
+	double width_in_pts, height_in_pts;
+
+	gog_graph_get_size (renderer->model, &width_in_pts, &height_in_pts);
+
+	switch (format) {
+		case GO_IMAGE_FORMAT_PNG:
+		case GO_IMAGE_FORMAT_JPG:
+			gog_renderer_pixbuf_update (prend, 
+						    width_in_pts * x_dpi / 72.0, 
+						    height_in_pts * y_dpi / 72.0, 1.0);
+			pixbuf = gog_renderer_pixbuf_get (prend);
+			if (pixbuf == NULL)
+				return FALSE;
+			format_info = go_image_get_format_info (format);
+			return gdk_pixbuf_save_to_callback (pixbuf,
+							    grp_gsf_gdk_pixbuf_save,
+							    output, format_info->name,
+							    NULL, NULL);
+			break;
+		default:
+			g_warning ("[GogRendererPixbuf:export_image] unsupported format");
+			return FALSE;
+	}
+}
+
 static void
 gog_renderer_pixbuf_class_init (GogRendererClass *rend_klass)
 {
@@ -963,6 +1011,7 @@ gog_renderer_pixbuf_class_init (GogRendererClass *rend_klass)
 	rend_klass->draw_marker	  	= gog_renderer_pixbuf_draw_marker;
 	rend_klass->get_text_OBR	= gog_renderer_pixbuf_get_text_OBR;
 	rend_klass->line_size		= gog_renderer_pixbuf_line_size;
+	rend_klass->export_image	= gog_renderer_pixbuf_export_image;
 }
 
 static void
@@ -980,6 +1029,15 @@ GSF_CLASS (GogRendererPixbuf, gog_renderer_pixbuf,
 	   gog_renderer_pixbuf_class_init, gog_renderer_pixbuf_init,
 	   GOG_RENDERER_TYPE)
 
+/**
+ * gog_renderer_pixbuf_get:
+ * @prend: a #GogRendererPixbuf
+ *
+ * An accessor to pixbuf cache.
+ *
+ * returns: current pixbuf cache.
+ **/
+
 GdkPixbuf *
 gog_renderer_pixbuf_get (GogRendererPixbuf *prend)
 {
@@ -988,62 +1046,18 @@ gog_renderer_pixbuf_get (GogRendererPixbuf *prend)
 	return prend->buffer;
 }
 
-#if 0 /* An initial non-working attempt to use different dpi to render
-	 different zooms */
-
-/* fontmaps are reasonably expensive use a cache to share them */
-static GHashTable *fontmap_cache = NULL; /* PangoFT2FontMap hashed by y_dpi */
-static gboolean
-cb_remove_entry (gpointer key, PangoFT2FontMap *value, PangoFT2FontMap *target)
-{
-	return value == target;
-}
-static void
-cb_map_is_gone (gpointer data, GObject *where_the_object_was)
-{
-	g_warning ("fontmap %p is gone",where_the_object_was);
-	g_hash_table_foreach_steal (fontmap_cache,
-		(GHRFunc) cb_remove_entry, where_the_object_was);
-}
-static void
-cb_weak_unref (GObject *fontmap)
-{
-	g_object_weak_unref (fontmap, cb_map_is_gone, NULL);
-}
-static PangoFT2FontMap *
-fontmap_from_cache (double x_dpi, double y_dpi)
-{
-	PangoFT2FontMap *fontmap = NULL;
-	int key_dpi = floor (y_dpi + .5);
-	gpointer key = GUINT_TO_POINTER (key_dpi);
-
-	if (fontmap_cache != NULL)
-		fontmap = g_hash_table_lookup (fontmap_cache, key);
-	else
-		fontmap_cache = g_hash_table_new_full (g_direct_hash, g_direct_equal,
-			NULL, (GDestroyNotify) cb_weak_unref);
-
-	if (fontmap == NULL) {
-		fontmap = PANGO_FT2_FONT_MAP (pango_ft2_font_map_new ());
-		pango_ft2_font_map_set_resolution (fontmap, x_dpi, y_dpi);
-		g_object_weak_ref (G_OBJECT (fontmap), cb_map_is_gone, NULL);
-		g_hash_table_insert (fontmap_cache, key, fontmap);
-	} else
-		g_object_ref (fontmap);
-
-	g_warning ("fontmap %d = %p", key_dpi, fontmap);
-	return fontmap;
-}
-#endif
-
 /**
- * gog_renderer_update :
- * @prend :
- * @w :
- * @h :
+ * gog_renderer_pixbuf_update:
+ * @prend: a #GogRendererPixbuf
+ * @w: requested width
+ * @h: requested height
+ * @zoom: request zoom
  *
- * Returns TRUE if the size actually changed.
+ * Asks for an update of pixbuf cache.
+ * 
+ * Returns: %TRUE if the size actually changed.
  **/
+
 gboolean
 gog_renderer_pixbuf_update (GogRendererPixbuf *prend, int w, int h, double zoom)
 {
@@ -1124,3 +1138,52 @@ gog_renderer_pixbuf_update (GogRendererPixbuf *prend, int w, int h, double zoom)
 
 	return redraw;
 }
+
+#if 0 /* An initial non-working attempt to use different dpi to render
+	 different zooms */
+
+/* fontmaps are reasonably expensive use a cache to share them */
+static GHashTable *fontmap_cache = NULL; /* PangoFT2FontMap hashed by y_dpi */
+static gboolean
+cb_remove_entry (gpointer key, PangoFT2FontMap *value, PangoFT2FontMap *target)
+{
+	return value == target;
+}
+static void
+cb_map_is_gone (gpointer data, GObject *where_the_object_was)
+{
+	g_warning ("fontmap %p is gone",where_the_object_was);
+	g_hash_table_foreach_steal (fontmap_cache,
+		(GHRFunc) cb_remove_entry, where_the_object_was);
+}
+static void
+cb_weak_unref (GObject *fontmap)
+{
+	g_object_weak_unref (fontmap, cb_map_is_gone, NULL);
+}
+static PangoFT2FontMap *
+fontmap_from_cache (double x_dpi, double y_dpi)
+{
+	PangoFT2FontMap *fontmap = NULL;
+	int key_dpi = floor (y_dpi + .5);
+	gpointer key = GUINT_TO_POINTER (key_dpi);
+
+	if (fontmap_cache != NULL)
+		fontmap = g_hash_table_lookup (fontmap_cache, key);
+	else
+		fontmap_cache = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+			NULL, (GDestroyNotify) cb_weak_unref);
+
+	if (fontmap == NULL) {
+		fontmap = PANGO_FT2_FONT_MAP (pango_ft2_font_map_new ());
+		pango_ft2_font_map_set_resolution (fontmap, x_dpi, y_dpi);
+		g_object_weak_ref (G_OBJECT (fontmap), cb_map_is_gone, NULL);
+		g_hash_table_insert (fontmap_cache, key, fontmap);
+	} else
+		g_object_ref (fontmap);
+
+	g_warning ("fontmap %d = %p", key_dpi, fontmap);
+	return fontmap;
+}
+#endif
+
