@@ -21,19 +21,24 @@
 
 #include <goffice/goffice-config.h>
 #include <goffice/graph/gog-renderer-impl.h>
+#ifdef GOFFICE_WITH_GTK
+#include <goffice/graph/gog-renderer-pixbuf.h>
+#endif
+#ifdef GOFFICE_WITH_CAIRO
+#include <goffice/graph/gog-renderer-cairo.h>
+#endif
+#include <goffice/graph/gog-renderer-svg.h>
 #include <goffice/graph/gog-style.h>
 #include <goffice/graph/gog-graph.h>
+#include <goffice/graph/gog-graph-impl.h>
 #include <goffice/graph/gog-view.h>
 #include <goffice/utils/go-units.h>
 #include <goffice/utils/go-font.h>
 #include <goffice/utils/go-math.h>
 
+#include <gsf/gsf.h>
 #include <gsf/gsf-impl-utils.h>
 
-/* We need to define an hair line width for the svg and gnome_print renderer. 
- * 0.24 pt is the dot size of a 300 dpi printer, if the plot is printed at scale 1:1 */
-#define GOG_RENDERER_HAIR_LINE_WIDTH	0.24
-
 enum {
 	RENDERER_PROP_0,
 	RENDERER_PROP_MODEL,
@@ -58,6 +63,11 @@ gog_renderer_finalize (GObject *obj)
 	go_line_vpath_dash_free (rend->outline_dash);
 	rend->outline_dash = NULL;
 
+	if (rend->grip_style != NULL) {
+		g_object_unref (rend->grip_style);
+		rend->grip_style = NULL;
+	}
+
 	if (rend->clip_stack != NULL) 
 		g_warning ("Missing calls to gog_renderer_pop_clip");
 
@@ -137,7 +147,7 @@ gog_renderer_get_property (GObject *obj, guint param_id,
 void
 gog_renderer_request_update (GogRenderer *renderer)
 {
-	g_return_if_fail (GOG_RENDERER (renderer) != NULL);
+	g_return_if_fail (IS_GOG_RENDERER (renderer));
 
 	if (renderer->needs_update)
 		return;
@@ -149,7 +159,7 @@ gog_renderer_request_update (GogRenderer *renderer)
 void
 gog_renderer_invalidate_size_requests (GogRenderer *rend)
 {
-	g_return_if_fail (GOG_RENDERER (rend) != NULL);
+	g_return_if_fail (IS_GOG_RENDERER (rend));
 
 	if (rend->view)
 		gog_renderer_request_update (rend);
@@ -180,7 +190,7 @@ gog_renderer_push_style (GogRenderer *rend, GogStyle const *style)
 	GogRendererClass *klass = GOG_RENDERER_GET_CLASS (rend);
 
 	g_return_if_fail (klass != NULL);
-	g_return_if_fail (GOG_STYLE (style) != NULL);
+	g_return_if_fail (IS_GOG_STYLE (style));
 
 	if (rend->cur_style != NULL)
 		rend->style_stack = g_slist_prepend (
@@ -272,10 +282,7 @@ gog_renderer_pop_clip (GogRenderer *rend)
 
 /**
  * gog_renderer_get_rectangle_vpath :
- * @x      : upper left corner
- * @y      : upper left corner
- * @width  : rectangle width
- * @height : rectangle height
+ * @rect:
  *
  * a utility routine to build a rectangle path.
  **/
@@ -393,13 +400,13 @@ gog_renderer_get_ring_wedge_bpath (double cx, double cy,
 
 /**
  * gog_renderer_draw_ring_wedge :
- * @renderer : #GogRenderer
- * @cx : center x coordinate
- * @cy : center y coordinate
- * @rx : x radius
- * @ry : y radius
- * @th0 : start arc angle
- * @th1 : stop arc angle
+ * @rend: #GogRenderer
+ * @cx: center x coordinate
+ * @cy: center y coordinate
+ * @rx_out: x radius
+ * @ry_out: y radius
+ * @th0: start arc angle
+ * @th1: stop arc angle
  *
  * a utility routine to draw an arc.
  **/
@@ -455,8 +462,8 @@ gog_renderer_draw_sharp_path (GogRenderer *rend, ArtVpath *path)
 
 /**
  * gog_renderer_draw_path :
- * @rend : #GogRenderer
- * @path  : #ArtVpath
+ * @rend: #GogRenderer
+ * @path: #ArtVpath
  *
  * Draws @path using the outline elements of the current style.
  **/
@@ -575,7 +582,8 @@ gog_renderer_draw_text (GogRenderer *rend, char const *text,
 /**
  * gog_renderer_draw_marker :
  * @rend : #GogRenderer
- * @pos  : #ArtPoint
+ * @x: X-coordinate
+ * @y: Y-coordinate
  **/
 void
 gog_renderer_draw_marker (GogRenderer *rend, double x, double y)
@@ -664,6 +672,7 @@ gog_renderer_class_init (GogRendererClass *renderer_klass)
 
 	renderer_klass->sharp_path = NULL;
 	renderer_klass->line_size = NULL;
+	renderer_klass->export_image = NULL;
 
 	g_object_class_install_property (gobject_klass, RENDERER_PROP_MODEL,
 		g_param_spec_object ("model", "model",
@@ -703,6 +712,8 @@ gog_renderer_init (GogRenderer *rend)
 	rend->font_watcher = g_cclosure_new_swap (G_CALLBACK (cb_font_removed),
 		rend, NULL);
 	go_font_cache_register (rend->font_watcher);
+
+	rend->grip_style = NULL;
 }
 
 GSF_CLASS (GogRenderer, gog_renderer,
@@ -710,11 +721,12 @@ GSF_CLASS (GogRenderer, gog_renderer,
 	   G_TYPE_OBJECT)
 
 /**
- * gog_renderer_draw_rectangle :
- * @renderer : #GogRenderer
- * @rect : #GogViewAllocation
+ * gog_renderer_draw_rectangle:
+ * @rend: a #GogRenderer
+ * @rect: position and extent of rectangle
+ *
+ * A utility routine to build a closed rectangle vpath.
  *
- * A utility routine to build a vpath in @rect.
  **/
 static void
 draw_rectangle (GogRenderer *rend, GogViewAllocation const *rect, gboolean sharp)
@@ -787,3 +799,206 @@ gog_renderer_pt2r (GogRenderer const *rend, double d)
 {
 	return d * rend->scale;
 }
+
+/**
+ * gog_renderer_draw_grip:
+ * @renderer : #GogRenderer
+ * @x : x position of grip
+ * @y : y position of grip
+ *
+ * Draw a grip, used for moving/resizing of objects.
+ **/
+void
+gog_renderer_draw_grip (GogRenderer *renderer, double x, double y) 
+{
+	GogViewAllocation rectangle;
+
+	rectangle.x = x - GOG_RENDERER_GRIP_SIZE;
+	rectangle.y = y - GOG_RENDERER_GRIP_SIZE;
+	rectangle.w = rectangle.h = 2 * GOG_RENDERER_GRIP_SIZE;
+
+	gog_renderer_draw_sharp_rectangle (renderer, &rectangle);
+}
+
+/**
+ * gog_renderer_push_selection_style:
+ * @renderer : #GogRenderer
+ *
+ * Push a style used for selection and grip rendering.
+ **/
+void
+gog_renderer_push_selection_style (GogRenderer *renderer)
+{
+	if (renderer->grip_style == NULL) {
+		renderer->grip_style = gog_style_new ();
+		renderer->grip_style->outline.dash_type = GO_LINE_SOLID;
+		renderer->grip_style->outline.width = 0.0;
+		renderer->grip_style->outline.color = 
+		renderer->grip_style->fill.pattern.back = 0xff000080;
+		renderer->grip_style->line.dash_type = GO_LINE_DOT;
+		renderer->grip_style->line.width = 0.0;
+		renderer->grip_style->line.color = 0x0000ffB0;
+		renderer->grip_style->fill.pattern.pattern = GO_PATTERN_SOLID;
+		renderer->grip_style->fill.type = GOG_FILL_STYLE_PATTERN;
+	}
+	gog_renderer_push_style (renderer, renderer->grip_style);
+}
+
+/**
+ * gog_renderer_new_for_pixbuf:
+ * @graph : graph model
+ *
+ * Creates a new renderer which can render into a pixbuf, and sets
+ * @graph as its model.
+ **/ 
+GogRenderer*
+gog_renderer_new_for_pixbuf (GogGraph *graph)
+{
+#ifdef GOFFICE_WITH_CAIRO
+	return g_object_new (GOG_RENDERER_CAIRO_TYPE, "model", graph, NULL);
+#else
+#ifdef GOFFICE_WITH_GTK
+	return g_object_new (GOG_RENDERER_PIXBUF_TYPE, "model", graph, NULL);
+#else
+	g_warning ("No cairo AND no gtk ???");
+	return NULL;
+#endif
+#endif
+}
+
+/**
+ * gog_renderer_update:
+ * @renderer: a #GogRenderer
+ * @w: requested width
+ * @h: requested height
+ * @zoom: requested zoom
+ *
+ * Requests a renderer update, only useful for pixbuf based renderer.
+ **/
+gboolean
+gog_renderer_update (GogRenderer *renderer, double w, double h, double zoom)
+{
+#ifdef GOFFICE_WITH_CAIRO
+	g_return_val_if_fail (IS_GOG_RENDERER_CAIRO (renderer), FALSE);
+	return gog_renderer_cairo_update (GOG_RENDERER_CAIRO (renderer), w, h, zoom);
+#else
+#ifdef GOFFICE_WITH_GTK
+	g_return_val_if_fail (IS_GOG_RENDERER_PIXBUF (renderer), FALSE);
+	return gog_renderer_pixbuf_update (GOG_RENDERER_PIXBUF (renderer), w, h, zoom);
+#else
+	g_warning ("No cairo AND no gtk ???");
+	return FALSE;
+#endif
+#endif
+}
+
+/** 
+ * gog_renderer_get_pixbuf:
+ * @renderer : #GogRenderer
+ *
+ * Returns current pixbuf buffer from a renderer that can render into a pixbuf.
+ **/
+GdkPixbuf*
+gog_renderer_get_pixbuf (GogRenderer *renderer)
+{
+#ifdef GOFFICE_WITH_CAIRO
+	g_return_val_if_fail (IS_GOG_RENDERER_CAIRO (renderer), NULL);
+     	return gog_renderer_cairo_get_pixbuf (GOG_RENDERER_CAIRO (renderer));
+#else
+#ifdef GOFFICE_WITH_GTK
+	g_return_val_if_fail (IS_GOG_RENDERER_PIXBUF (renderer), NULL);
+	return gog_renderer_pixbuf_get (GOG_RENDERER_PIXBUF (renderer));
+#else
+	g_warning ("No cairo AND no gtk ???");
+	return NULL;
+#endif
+#endif
+}
+
+/**
+ * gog_renderer_new_for_format:
+ * @graph: a #GogGraph
+ * @format: image format
+ * 
+ * Creates a new #GogRenderer which is capable of export to @format.
+ *
+ * returns: a new #GogRenderer.
+ **/
+
+GogRenderer *
+gog_renderer_new_for_format (GogGraph *graph, GOImageFormat format)
+{
+	GType type = G_TYPE_INVALID;
+	
+	switch (format) {
+		case GO_IMAGE_FORMAT_PNG:
+		case GO_IMAGE_FORMAT_JPG:
+#ifdef GOFFICE_WITH_CAIRO
+			type = GOG_RENDERER_CAIRO_TYPE;
+#else
+#ifdef GOFFICE_WITH_GTK
+			type = GOG_RENDERER_PIXBUF_TYPE;
+			g_warning ("No cairo AND no gtk ???");
+			return NULL;
+#endif
+#endif
+			break;
+		case GO_IMAGE_FORMAT_SVG:
+#ifdef GOG_RENDERER_CAIRO_WITH_SVG
+			type = GOG_RENDERER_CAIRO_TYPE;
+#else
+			type = GOG_RENDERER_SVG_TYPE;
+#endif
+			break;
+		case GO_IMAGE_FORMAT_PDF:
+#ifdef GOG_RENDERER_CAIRO_WITH_PDF
+			type = GOG_RENDERER_CAIRO_TYPE;
+#endif
+			break;
+		case GO_IMAGE_FORMAT_PS:
+#ifdef GOG_RENDERER_CAIRO_WITH_PS
+			type = GOG_RENDERER_CAIRO_TYPE;
+#endif
+			break;
+		default:
+			break;
+	}
+	
+	if (type == G_TYPE_INVALID)
+		return NULL;
+	
+	return g_object_new (type, "model", graph, NULL);	
+}
+
+/**
+ * gog_renderer_export_image:
+ * @renderer: a #GogRenderer
+ * @format: image format for export
+ * @output: a #GsfOutput stream
+ * @x_dpi: x resolution of exported graph
+ * @y_dpi: y resolution of exported graph
+ *
+ * Exports an image of @graph in given @format, writing results in a #GsfOutput stream.
+ * If export format type is a bitmap one, it computes image size with x_dpi, y_dpi and 
+ * @graph size (see gog_graph_get_size()).
+ *
+ * returns: %TRUE if export succeed.
+ **/
+
+gboolean
+gog_renderer_export_image (GogRenderer *renderer, GOImageFormat format,
+			   GsfOutput *output, double x_dpi, double y_dpi)
+{
+	GogRendererClass *klass;
+
+	g_return_val_if_fail (IS_GOG_RENDERER (renderer), FALSE);
+       
+	gog_graph_force_update (renderer->model);
+
+	klass = GOG_RENDERER_GET_CLASS (renderer);
+
+	if (klass->export_image != NULL)
+		return (klass->export_image) (renderer, format, output, x_dpi, y_dpi);
+
+	return FALSE;
+}
