@@ -24,23 +24,12 @@
 #include <goffice/app/go-cmd-context.h>
 #include <goffice/utils/go-file.h>
 #include <goffice/goffice-priv.h>
-#include <gtk/gtkalignment.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkmessagedialog.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkdialog.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkbbox.h>
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtkfilechooserdialog.h>
-#include <gtk/gtkicontheme.h>
+#include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <atk/atkrelation.h>
 #include <atk/atkrelationset.h>
 #include <glib/gi18n-lib.h>
+#include <glib/gstdio.h>
 #include <gsf/gsf-input-stdio.h>
 #include <gsf/gsf-input-textline.h>
 
@@ -57,45 +46,13 @@
 #define W_OK 2
 #endif
 
-/* g_access would be nice here.  */
-static int
-go_access (const char *filename, int what)
-{
+#ifndef HAVE_G_ACCESS
 #ifdef G_OS_WIN32
-	int retval;
-	int save_errno;
-	if (G_WIN32_HAVE_WIDECHAR_API ()) {
-		wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
-		if (wfilename == NULL) {
-			errno = EINVAL;
-			return -1;
-		}
-
-		retval = _waccess (wfilename, what);
-		save_errno = errno;
-
-		g_free (wfilename);
-      
-		errno = save_errno;
-	} else {
-		gchar *cp_filename = g_locale_from_utf8 (filename, -1, NULL, NULL, NULL);
-		if (cp_filename == NULL) {
-			errno = EINVAL;
-			return -1;
-		}
-
-		retval = _access (cp_filename, what);
-		save_errno = errno;
-
-		g_free (cp_filename);
-
-		errno = save_errno;
-	}
-	return retval;
+#error "A glib with g_access is required for Win32"
 #else
-	return access (filename, what);
+#define g_access access
+#endif
 #endif
-}
 
 /* ------------------------------------------------------------------------- */
 
@@ -124,11 +81,11 @@ go_gtk_button_new_with_stock (char const *text, char const* stock_id)
 }
 
 /**
- * go_gtk_dialog_add_button
- * dialog : dialog you want to add a button
- * text : button label
- * stock_id : stock icon id
- * response_id : respond id when button clicked
+ * go_gtk_dialog_add_button :
+ * @dialog: dialog you want to add a button
+ * @text: button label
+ * @stock_id: stock icon id
+ * @response_id: respond id when button clicked
  *
  * Creates and adds a button with stock image to the action area of an existing dialog.
  * Code from gedit
@@ -553,47 +510,110 @@ go_gtk_select_image (GtkWindow *toplevel, const char *initial)
 	return uri;
 }
 
-char *
-gui_get_image_save_info (GtkWindow *toplevel, GSList *formats,
-			 GOImageType const **ret_format)
+/**
+ * gui_get_image_save_info:
+ * @toplevel: a #GtkWindow
+ * @supported_formats: a #GSList of supported file formats
+ * @ret_format: default file format
+ * @resolution: export resolution
+ *
+ * Opens a file chooser and let user choose file URI and format in a list of
+ * supported ones.
+ *
+ * returns: file URI string, file #GOImageFormat stored in @ret_format, and
+ * export resolution in @resolution.
+ **/
+
+typedef struct {
+	char *uri;
+	double resolution;
+	gboolean is_expanded;
+	GOImageFormat format;
+} SaveInfoState;
+
+static void
+save_info_state_free (SaveInfoState *state) 
 {
-	GOImageType const *sel_format = NULL;
+	g_free (state->uri);
+	g_free (state);
+}
+
+char *
+gui_get_image_save_info (GtkWindow *toplevel, GSList *supported_formats,
+			 GOImageFormat *ret_format, double *resolution)
+{
+	GOImageFormat format;
+	GOImageFormatInfo const *format_info;
 	GtkComboBox *format_combo = NULL;
-	char *uri = NULL;
 	GtkFileChooser *fsel = gui_image_chooser_new (TRUE);
+	GtkWidget *expander = NULL;
+	GtkWidget *resolution_spin = NULL;
+	GladeXML *gui;
+	SaveInfoState *state;
+	const char *key = "gui_get_image_save_info";
+	char *uri = NULL;
+
+	state = g_object_get_data (G_OBJECT (toplevel), key);
+	if (state == NULL) {
+		state = g_new (SaveInfoState, 1);
+		g_return_val_if_fail (state != NULL, NULL);
+		state->uri = NULL;
+		state->resolution = 150.0;
+		state->is_expanded = FALSE;
+		state->format = GO_IMAGE_FORMAT_SVG;
+		g_object_set_data_full (G_OBJECT (toplevel), key,
+					state, (GDestroyNotify) save_info_state_free);
+	}
 
 	g_object_set (G_OBJECT (fsel), "title", _("Save as"), NULL);
 
-	/* Make format chooser */
-	if (formats && ret_format) {
-		GtkWidget *label;
-		GtkWidget *box = gtk_hbox_new (FALSE, 5);
-		GSList *l;
-		int i;
+	gui = go_libglade_new ("go-image-save-dialog-extra.glade", 
+			       "image_save_dialog_extra", 
+			       GETTEXT_PACKAGE, NULL);
+	if (gui != NULL) {
+		GtkWidget *widget;
 
-		format_combo = GTK_COMBO_BOX (gtk_combo_box_new_text ());
-		if (*ret_format)
-			sel_format = *ret_format;
-		for (l = formats, i = 0; l != NULL; l = l->next, i++) {
-#warning we must find a better solution for translating strings not in goffice domain
-			gtk_combo_box_append_text
-				(format_combo,
-				 gettext (((GOImageType *) (l->data))->desc));
-			if (l->data == (void *)sel_format)
-				gtk_combo_box_set_active (format_combo, i);
+		/* Format selection UI */
+		if (supported_formats != NULL && ret_format != NULL) {
+			int i;
+			GSList *l;
+			format_combo = GTK_COMBO_BOX (glade_xml_get_widget (gui, "format_combo"));
+			for (l = supported_formats, i = 0; l != NULL; l = l->next, i++) {
+				format = GPOINTER_TO_UINT (l->data);
+				format_info = go_image_get_format_info (format);
+				gtk_combo_box_append_text (format_combo, _(format_info->desc)); 
+				if (format == state->format)
+					gtk_combo_box_set_active (format_combo, i);
+			}
+			if (gtk_combo_box_get_active (format_combo) < 0)
+				gtk_combo_box_set_active (format_combo, 0);
+		} else {
+			widget = glade_xml_get_widget (gui, "file_type_box");
+			gtk_widget_hide (widget);
 		}
-		if (gtk_combo_box_get_active (format_combo) < 0)
-			gtk_combo_box_set_active (format_combo, 0);
 
-		label = gtk_label_new_with_mnemonic (_("File _type:"));
-		gtk_box_pack_start (GTK_BOX (box), label, FALSE, TRUE, 0);
-		gtk_box_pack_start (GTK_BOX (box),  GTK_WIDGET (format_combo),
-				    TRUE, TRUE, 0);
-		gtk_label_set_mnemonic_widget (GTK_LABEL (label),
-					       GTK_WIDGET (format_combo));
-		gtk_file_chooser_set_extra_widget (fsel, box);
+		/* Export setting expander */
+		expander = glade_xml_get_widget (gui, "export_expander");
+		if (resolution != NULL) {
+			gtk_expander_set_expanded (GTK_EXPANDER (expander), state->is_expanded);
+			resolution_spin = glade_xml_get_widget (gui, "resolution_spin");
+			gtk_spin_button_set_value (GTK_SPIN_BUTTON (resolution_spin), state->resolution);
+		} else 
+			gtk_widget_hide (expander);
+
+		if (resolution != NULL && supported_formats != NULL && ret_format != NULL) {
+			widget = glade_xml_get_widget (gui, "image_save_dialog_extra");
+			gtk_file_chooser_set_extra_widget (fsel, widget);
+		}
+
+		g_object_unref (G_OBJECT (gui));
 	}
 
+	if (state->uri != NULL) {
+		gtk_file_chooser_set_uri (fsel, state->uri);
+		gtk_file_chooser_unselect_all (fsel);
+	}
+	
 	/* Show file selector */
  loop:
 	if (!go_gtk_file_sel_dialog (toplevel, GTK_WIDGET (fsel)))
@@ -602,9 +622,11 @@ gui_get_image_save_info (GtkWindow *toplevel, GSList *formats,
 	if (format_combo) {
 		char *new_uri = NULL;
 
-		sel_format = g_slist_nth_data (
-			formats, gtk_combo_box_get_active (format_combo));
-		if (!go_url_check_extension (uri, sel_format->ext, &new_uri) &&
+		format = GPOINTER_TO_UINT (g_slist_nth_data 
+			(supported_formats, 
+			 gtk_combo_box_get_active (format_combo)));
+		format_info = go_image_get_format_info (format);
+		if (!go_url_check_extension (uri, format_info->ext, &new_uri) &&
 		    !go_gtk_query_yes_no (GTK_WINDOW (fsel), TRUE,
 		     _("The given file extension does not match the"
 		       " chosen file type. Do you want to use this name"
@@ -617,7 +639,7 @@ gui_get_image_save_info (GtkWindow *toplevel, GSList *formats,
 			g_free (uri);
 			uri = new_uri;
 		}
-		*ret_format = sel_format;
+		*ret_format = format;
 	}
 	if (!go_gtk_url_is_writeable (GTK_WINDOW (fsel), uri, TRUE)) {
 		g_free (uri);
@@ -625,72 +647,20 @@ gui_get_image_save_info (GtkWindow *toplevel, GSList *formats,
 		goto loop;
 	}
  out:
-	gtk_widget_destroy (GTK_WIDGET (fsel));
-	return uri;
-}
-
-char *
-go_mime_to_image_format (char const *mime_type)
-{
- 	guint i;
-	const char *suffix;
-	const char* exceptions[] = {
-		"svg+xml", "svg",
-		"x-wmf", "wmf",
-		"x-emf", "emf",
-	};
-
-	if (strncmp (mime_type, "image/", 6) != 0)
-		return NULL;
-	suffix = mime_type + 6;
-	for (i = 0; i < G_N_ELEMENTS (exceptions); i +=2)
-		if (strcmp (suffix, exceptions[i]) == 0)
-			return g_strdup (exceptions[i+1]);
-
-	return g_strdup (suffix);
-}
-
-char *
-go_image_format_to_mime (char const *format)
-{
- 	guint i;
-	GSList *ptr, *pixbuf_fmts;
-	GdkPixbufFormat *pfmt;
-	gchar *name;
-	char *ret = NULL;
-	int cmp;
-	gchar **mimes;
-
-	const char* formats[] = {
-		"svg", "image/svg,image/svg+xml",
-		"wmf", "x-wmf",
-		"emf", "x-emf",
-	};
-	
-	if (format == NULL)
-		return NULL;
-
-	for (i = 0; i < G_N_ELEMENTS (formats); i +=2)
-		if (strcmp (format, formats[i]) == 0)
-			return g_strdup (formats[i+1]);
-
-	/* Not a format we have special knowledge about - ask gdk-pixbuf */
-	pixbuf_fmts = gdk_pixbuf_get_formats ();
-	for (ptr = pixbuf_fmts; ptr != NULL; ptr = ptr->next) {
-		pfmt = (GdkPixbufFormat *)ptr->data;
-		name = gdk_pixbuf_format_get_name (pfmt);
-		cmp = strcmp (format, name);
-		g_free (name);
-		if (cmp == 0) {
-			mimes = gdk_pixbuf_format_get_mime_types (pfmt);
-			ret = g_strjoinv (",", mimes);
-			g_strfreev (mimes);
-			break;
+	if (uri != NULL) {
+		g_free (state->uri);
+		state->uri = g_strdup (uri);
+		state->format = *ret_format;
+		if (resolution != NULL) {
+			state->is_expanded = gtk_expander_get_expanded (GTK_EXPANDER (expander));
+			*resolution =  gtk_spin_button_get_value (GTK_SPIN_BUTTON (resolution_spin));
+			state->resolution = *resolution;
 		}
 	}
-	g_slist_free (pixbuf_fmts);
 
-	return ret;
+	gtk_widget_destroy (GTK_WIDGET (fsel));
+
+	return uri;
 }
 
 static void
@@ -726,15 +696,16 @@ typedef struct {
 	char const *link;
 } CBHelpPaths;
 
-#ifdef WITH_GNOME
+#ifdef GOFFICE_WITH_GNOME
 #include <libgnome/gnome-help.h>
 #elif defined(G_OS_WIN32)
-#include <goffice/utils/win32-stub.h>
+#include <windows.h>
+#include <htmlhelp.h>
 #endif
 static void
 go_help_display (CBHelpPaths const *paths)
 {
-#ifdef WITH_GNOME
+#ifdef GOFFICE_WITH_GNOME
 	gnome_help_display (paths->app, paths->link, NULL);
 #elif defined(G_OS_WIN32)
 	static GHashTable* context_help_map = NULL;
@@ -785,7 +756,7 @@ go_help_display (CBHelpPaths const *paths)
 		gchar *path = g_build_filename (paths->data_dir, "doc", "C", chmfile, NULL);
 
 		g_free (chmfile);
-		if (!HtmlHelp_ (GetDesktopWindow (), path, HH_HELP_CONTEXT, id))
+		if (!HtmlHelp (GetDesktopWindow (), path, HH_HELP_CONTEXT, id))
 			go_gtk_notice_dialog (NULL, GTK_MESSAGE_ERROR, "Failed to spawn HtmlHelp");
 		g_free (path);
 	}
@@ -851,7 +822,7 @@ go_gtk_url_is_writeable (GtkWindow *parent, char const *uri,
 		go_gtk_notice_dialog (parent, GTK_MESSAGE_ERROR,
 				      _("%s\nis a directory name"), uri);
 		result = FALSE;
-	} else if (go_access (filename, W_OK) != 0 && errno != ENOENT) {
+	} else if (g_access (filename, W_OK) != 0 && errno != ENOENT) {
 		go_gtk_notice_dialog (parent, GTK_MESSAGE_ERROR,
 				      _("You do not have permission to save to\n%s"),
 				      uri);
@@ -881,17 +852,6 @@ go_gtk_url_is_writeable (GtkWindow *parent, char const *uri,
 	return result;
 }
 
-static gint
-cb_modal_dialog_keypress (GtkWidget *w, GdkEventKey *e)
-{
-	if(e->keyval == GDK_Escape) {
-		gtk_dialog_response (GTK_DIALOG (w), GTK_RESPONSE_CANCEL);
-		return TRUE;
-	}
-
-	return FALSE;
-}
-
 /**
  * go_gtk_dialog_run
  *
@@ -910,13 +870,14 @@ go_gtk_dialog_run (GtkDialog *dialog, GtkWindow *parent)
 		go_gtk_window_set_transient (parent, GTK_WINDOW (dialog));
 	}
 
-	g_signal_connect (G_OBJECT (dialog),
-		"key-press-event",
-		G_CALLBACK (cb_modal_dialog_keypress), NULL);
+	g_object_ref (dialog);
 
 	while ((result = gtk_dialog_run (dialog)) >= 0)
 	       ;
 	gtk_widget_destroy (GTK_WIDGET (dialog));
+
+	g_object_unref (dialog);
+
 	return result;
 }
 
@@ -970,8 +931,6 @@ go_gtk_notice_nonmodal_dialog (GtkWindow *parent, GtkWidget **ref,
 		G_CALLBACK (gtk_widget_destroyed), ref);
 
 	gtk_widget_show (dialog);
-
-	return;
 }
 
 gboolean
@@ -1012,3 +971,92 @@ go_pixbuf_new_from_file (char const *filename)
 	g_free (path);
 	return pixbuf;
 }
+
+#ifndef HAVE_GTK_DIALOG_GET_RESPONSE_FOR_WIDGET
+/* This is public from 2.8 onwards.   */
+static gint
+gtk_dialog_get_response_for_widget (GtkDialog *dialog, GtkWidget *widget)
+{
+	const struct {
+		gint response_id;
+	} *rd = g_object_get_data (G_OBJECT (widget),
+				   "gtk-dialog-response-data");
+	if (!rd)
+		return GTK_RESPONSE_NONE;
+	else
+		return rd->response_id;
+}
+#endif
+
+
+/**
+ * go_dialog_guess_alternative_button_order:
+ * @dialog :
+ *
+ * This function inspects the buttons in the dialog and comes up
+ * with a reasonable alternative dialog order.
+ **/
+void
+go_dialog_guess_alternative_button_order (GtkDialog *dialog)
+{
+	GList *children, *tmp;
+	int i, nchildren;
+	int *new_order;
+	int i_yes = -1, i_no = -1, i_ok = -1, i_cancel = -1, i_apply = -1;
+	gboolean again;
+	gboolean any = FALSE;
+	int loops = 0;
+
+	children = gtk_container_get_children (GTK_CONTAINER (dialog->action_area));
+	if (!children)
+		return;
+
+	nchildren = g_list_length (children);
+	new_order = g_new (int, nchildren);
+
+	for (tmp = children, i = 0; tmp; tmp = tmp->next, i++) {
+		GtkWidget *child = tmp->data;
+		int res = gtk_dialog_get_response_for_widget (dialog, child);
+		new_order[i] = res;
+		switch (res) {
+		case GTK_RESPONSE_YES: i_yes = i; break;
+		case GTK_RESPONSE_NO: i_no = i; break;
+		case GTK_RESPONSE_OK: i_ok = i; break;
+		case GTK_RESPONSE_CANCEL: i_cancel = i; break;
+		case GTK_RESPONSE_APPLY: i_apply = i; break;
+		}
+	}
+	g_list_free (children);
+
+#define MAYBE_SWAP(ifirst,ilast)				\
+	if (ifirst >= 0 && ilast >= 0 && ifirst > ilast) {	\
+		int tmp;					\
+								\
+		tmp = new_order[ifirst];			\
+		new_order[ifirst] = new_order[ilast];		\
+		new_order[ilast] = tmp;				\
+								\
+		tmp = ifirst;					\
+		ifirst = ilast;					\
+		ilast = tmp;					\
+								\
+		again = TRUE;					\
+		any = TRUE;					\
+	}
+
+	do {
+		again = FALSE;
+		MAYBE_SWAP (i_yes, i_no);
+		MAYBE_SWAP (i_ok, i_cancel);
+		MAYBE_SWAP (i_cancel, i_apply);
+		MAYBE_SWAP (i_no, i_cancel);
+	} while (again && ++loops < 2);
+
+#undef MAYBE_SWAP
+
+	if (any)
+		gtk_dialog_set_alternative_button_order_from_array (dialog,
+								    nchildren,
+								    new_order);
+	g_free (new_order);
+}
