@@ -24,7 +24,6 @@
 #include <goffice/goffice-config.h>
 #include "go-format.h"
 #include "format-impl.h"
-#include "go-format-match.h"
 #include "go-color.h"
 #include "datetime.h"
 #include "go-glib-extras.h"
@@ -123,68 +122,6 @@ static double beyond_precision;
 static long double beyond_precisionl;
 #endif
 
-/* FIXME : use nl_langinfo
- * DAY_{1-7} (LC_TIME)
- * ABDAY_{1-7} (LC_TIME)
- * MON_{1-12} (LC_TIME)
- * ABMON_{1-12} (LC_TIME)
- *
- * The down side is that we can not guarantee the case
- **/
-char const * const day_short [] = {
-	N_("*Sun"),
-	N_("*Mon"),
-	N_("*Tue"),
-	N_("*Wed"),
-	N_("*Thu"),
-	N_("*Fri"),
-	N_("*Sat"),
-	NULL,
-};
-
-char const * const day_long [] = {
-	N_("Sunday"),
-	N_("Monday"),
-	N_("Tuesday"),
-	N_("Wednesday"),
-	N_("Thursday"),
-	N_("Friday"),
-	N_("Saturday"),
-	NULL
-};
-
-char const * const month_short [] = {
-	N_("*Jan"),
-	N_("*Feb"),
-	N_("*Mar"),
-	N_("*Apr"),
-	N_("*May"),
-	N_("*Jun"),
-	N_("*Jul"),
-	N_("*Aug"),
-	N_("*Sep"),
-	N_("*Oct"),
-	N_("*Nov"),
-	N_("*Dec"),
-	NULL
-};
-
-char const * const month_long [] = {
-	N_("January"),
-	N_("February"),
-	N_("March"),
-	N_("April"),
-	N_("May"),
-	N_("June"),
-	N_("July"),
-	N_("August"),
-	N_("September"),
-	N_("October"),
-	N_("November"),
-	N_("December"),
-	NULL
-};
-
 static GOColor lookup_color (char const *str, char const *end);
 
 char const *
@@ -447,7 +384,7 @@ append_year (GString *string, gchar const *format, struct tm const *time_split)
 static int
 append_month (GString *string, int n, struct tm const *time_split)
 {
-	int month = time_split->tm_mon + 1;
+	GDateMonth month = time_split->tm_mon + 1;
 
 	if (n == 1) {
 		g_string_append_printf (string, "%d", month);
@@ -460,16 +397,25 @@ append_month (GString *string, int n, struct tm const *time_split)
 	}
 
 	if (n == 3) {
-		g_string_append (string, _(month_short[month - 1]) + 1);
+		char *s = go_date_month_name (month, TRUE);
+		g_string_append (string, s);
+		g_free (s);
 		return 3;
 	}
 
 	if (n == 5) {
-		g_string_append_c (string, _(month_short[month - 1])[1]);
+		char *s = go_date_month_name (month, TRUE);
+		if (s[0]) g_string_append_unichar (string, g_utf8_get_char (s));
+		g_free (s);
 		return 5;
 	}
-	g_string_append (string, _(month_long[month - 1]));
-	return 4;
+
+	{
+		char *s = go_date_month_name (month, FALSE);
+		g_string_append (string, s);
+		g_free (s);
+		return 4;
+	}
 }
 
 /*
@@ -491,13 +437,21 @@ append_day (GString *string, gchar const *format, struct tm const *time_split)
 
 	if (format[3] != 'd' && format[3] != 'D') {
 		/* Note: day-of-week.  */
-		g_string_append (string, _(day_short[time_split->tm_wday]) + 1);
+		GDateWeekday wd = (time_split->tm_wday + 6) % 7 + 1;
+		char *s = go_date_weekday_name (wd, TRUE);
+		g_string_append (string, s);
+		g_free (s);
 		return 3;
 	}
 
-	/* Note: day-of-week.  */
-	g_string_append (string, _(day_long[time_split->tm_wday]));
-	return 4;
+	{
+		/* Note: day-of-week.  */
+		GDateWeekday wd = (time_split->tm_wday + 6) % 7 + 1;
+		char *s = go_date_weekday_name (wd, FALSE);
+		g_string_append (string, s);
+		g_free (s);
+		return 4;
+	}
 }
 
 static void
@@ -588,8 +542,6 @@ format_entry_ctor (GOFormat *container)
 	entry->elapsed_time = FALSE;
 	entry->want_am_pm = entry->has_fraction = FALSE;
 	entry->go_color = 0;
-	entry->regexp_str = NULL;
-	entry->match_tags = NULL;
 
 	/* symbolic failure */
 	g_return_val_if_fail (container != NULL, entry);
@@ -604,7 +556,6 @@ static void
 format_entry_dtor (gpointer data, gpointer user_data)
 {
 	GOFormatElement *entry = data;
-	format_match_release (entry);
 	g_free ((char *)entry->format);
 	g_free (entry);
 }
@@ -621,7 +572,6 @@ format_entry_set_fmt (GOFormatElement *entry,
 		? g_strndup (begin, end - begin)
 		: g_strdup ((entry->go_color || entry->restriction_type != '*')
 			    ? "General" : "");
-	format_match_create (entry);
 }
 #endif
 
@@ -676,13 +626,19 @@ format_compile (GOFormat *format)
 				}
 			} else if (*begin == '=') {
 				entry->restriction_type = '=';
+			} else if (*begin == '$') {
+				/* A currency.  */
+				if (NULL == real_start)
+					real_start = fmt;
+				fmt = end;
+				continue;
 			} else {
-				if (begin[1] == ']' &&
-				    (*begin == 'h' || *begin == 'H' ||
+				if ((*begin == 'h' || *begin == 'H' ||
 				     *begin == 'm' || *begin == 'M' ||
-				     *begin == 's' || *begin == 'S'))
+				     *begin == 's' || *begin == 'S') &&
+				    begin[1] == ']')
 					entry->elapsed_time = TRUE;
-				else if (*begin != '$' && entry->go_color == 0) {
+				else if (entry->go_color == 0) {
 					entry->go_color = lookup_color (begin, end);
 					/* Only the first colour counts */
 					if (0 != entry->go_color) {
@@ -698,6 +654,7 @@ format_compile (GOFormat *format)
 
 			/* fall back on 0 for errors */
 			errno = 0;
+			/* FIXME: ->restriction_value should probably be a long double.  */
 			entry->restriction_value = STRTO (begin, (char **)&end);
 			if (errno == ERANGE || begin == end)
 				entry->restriction_value = 0.;
@@ -957,6 +914,128 @@ SUFFIX(go_render_number) (GString *result,
 		g_string_append_c (result, ' ');
 }
 
+/* TODO: this function currently does not take into account:
+ * 	- left_req > 1
+ * 	- mixed right_optional and right_req. It always uses g format
+ * 	  for right_optional > 0 */
+
+static void
+SUFFIX(go_render_number_scientific) (GString *result, 
+				      DOUBLE value, 
+				      GONumberFormat const *info)
+{
+	char const *mantissa_sign = "";
+	char const *exponent_sign = "";
+	DOUBLE mantissa;
+	DOUBLE epsilon;
+	int exponent;
+	int exponent_digit_nbr = info->exponent_digit_nbr;
+	int exponent_step = info->left_req + info->left_optional;
+	int precision = info->right_req + info->right_optional;
+	int format_precision;
+	int i;
+	gboolean show_exponent_sign = info->exponent_show_sign;
+	gboolean lower_e = info->exponent_lower_e;
+	gboolean use_markup = info->use_markup;
+	gboolean use_g_format = info->right_optional > 0;
+	gboolean is_zero = FALSE;
+
+	if (exponent_step < 1)
+		exponent_step = 1;
+
+	if (value < 0.) {
+		mantissa_sign = "-";
+		value = -value;
+	}
+
+	if (precision < 0)
+		precision = 0;
+
+	epsilon = SUFFIX(pow) (10, -precision) / 2.0; 
+
+	if (SUFFIX(go_sub_epsilon) (value) <= 0) {
+		exponent = 0;
+		is_zero = TRUE;
+	} else {
+		exponent = (int) SUFFIX(floor) (SUFFIX(log10) (value));
+		if (exponent >= 0)
+			exponent = (exponent / exponent_step) * exponent_step;
+		else
+			exponent = (exponent + 1 - exponent_step) / exponent_step * exponent_step;
+	}
+
+	mantissa = value * SUFFIX(pow) (10, -exponent);
+
+	/* Here we try to avoid to display 10.00E-01 for 1.0-epsilon */
+	if (!is_zero && 
+	    SUFFIX(log10) (mantissa + epsilon) >= (DOUBLE) exponent_step) {
+		exponent += exponent_step;
+		mantissa = value * SUFFIX(pow) (10, -exponent);
+	}
+	
+	if (exponent < 0 ||
+	    (exponent == 0 && mantissa < 1.0)) {
+		exponent_sign = "-";
+		exponent = -exponent;
+	} else if (show_exponent_sign && !use_markup)
+		exponent_sign = "+";
+
+	/* Calculate precision for g format. With g, precision is the maximum number 
+	 * of displayed digits, so we have to add to requested precision the actual 
+	 * number of left digits in mantissa */
+	format_precision = precision + 
+		((use_g_format && !is_zero) ? (MAX ((int) SUFFIX(floor) (SUFFIX(log10) (mantissa)) + 1, 1)) : 0);
+
+	if (exponent == 0 && exponent_digit_nbr == 0) {
+		g_string_append_printf (result, 
+					use_g_format ? "%s%.*" FORMAT_G : "%s%.*" FORMAT_f, 
+					mantissa_sign, 
+					format_precision, mantissa); 
+	} else {
+		if (use_markup) {
+			/* Don't render mantissa when it's almost 1.0 and no left digit is required,
+			 * taking into account precision */ 
+			if ((SUFFIX(fabs) (mantissa - 1) < epsilon) &&
+			    info->left_req < 1)
+				g_string_append_printf (result, "%s10<sup>%s%d</sup>", 
+							mantissa_sign, 
+							exponent_sign,
+							exponent); 
+			else {
+				g_string_append_printf (result, 
+							use_g_format ? "%s%.*" FORMAT_G : "%s%.*" FORMAT_f,
+							mantissa_sign, 
+							format_precision, mantissa);
+				g_string_append_unichar (result, 0x00D7); /* multiplication sign */
+				g_string_append_printf (result, "10<sup>%s%d</sup>", 
+							exponent_sign,
+							exponent); 
+			}
+		} else {
+			char const* format;
+			int exponent_start;
+			int length;
+
+			if (use_g_format)
+				format = lower_e ? "%s%.*" FORMAT_G "e%s%n%d" : "%s%.*" FORMAT_G "E%s%n%d";
+			else
+				format = lower_e ? "%s%.*" FORMAT_f "e%s%n%d" : "%s%.*" FORMAT_f "E%s%n%d";
+
+			g_string_append_printf (result, format, 
+						mantissa_sign,
+						format_precision, mantissa, 
+						exponent_sign,
+						&exponent_start,
+						exponent);
+
+			length = result->len - exponent_start;
+			if (length < exponent_digit_nbr)
+				for (i = 0; i < exponent_digit_nbr - length; i++)
+					g_string_insert_c (result, exponent_start, '0');
+		}
+	}
+}
+
 static void
 SUFFIX(do_render_number) (DOUBLE number, GONumberFormat *info, GString *result)
 {
@@ -981,7 +1060,10 @@ SUFFIX(do_render_number) (DOUBLE number, GONumberFormat *info, GString *result)
 		decimal_point);
 #endif
 
-	SUFFIX(go_render_number) (result, info->scale * number, info);
+	if (info->exponent_seen)
+		SUFFIX(go_render_number_scientific) (result, number, info);
+	else 
+		SUFFIX(go_render_number) (result, info->scale * number, info);
 }
 
 static DOUBLE
@@ -1186,17 +1268,26 @@ go_format_as_scientific (GOFormatDetails const *fmt)
 {
 	GString *str;
 	GOFormat *gf;
+	int i;
 
 	g_return_val_if_fail (fmt->num_decimals >= 0, NULL);
 	g_return_val_if_fail (fmt->num_decimals <= NUM_ZEROS, NULL);
 
 	str = g_string_new (NULL);
-	g_string_append_c (str, '0');
+	for (i = 0; i < fmt->exponent_step - 1; i++)
+		g_string_append_c (str, '#');
+	if (fmt->simplify_mantissa)
+		g_string_append_c (str, '#');
+	else
+		g_string_append_c (str, '0');
 	if (fmt->num_decimals > 0) {
 		g_string_append_c (str, '.');
 		g_string_append_len (str, zeros, fmt->num_decimals);
 	}
-	g_string_append (str, "E+00");
+	if (fmt->use_markup)
+		g_string_append (str, "EE0");
+	else
+		g_string_append (str, "E+00");
 
 	gf = go_format_new_from_XL (str->str, FALSE);
 	g_string_free (str, TRUE);
@@ -1651,8 +1742,12 @@ SUFFIX(go_format_number) (GString *result,
 
 		case '#':
 			can_render_number = TRUE;
-			if (info.decimal_separator_seen)
-				info.right_optional++;
+			if (!info.exponent_seen) { 
+				if (info.decimal_separator_seen)
+					info.right_optional++;
+				else 
+					info.left_optional++;
+			}
 			break;
 
 		case '?':
@@ -1665,13 +1760,17 @@ SUFFIX(go_format_number) (GString *result,
 
 		case '0':
 			can_render_number = TRUE;
-			if (info.decimal_separator_seen){
-				info.right_req++;
-				info.right_allowed++;
-				info.right_spaces++;
-			} else {
-				info.left_spaces++;
-				info.left_req++;
+			if (info.exponent_seen) 
+				info.exponent_digit_nbr++;
+			else {
+				if (info.decimal_separator_seen){
+					info.right_req++;
+					info.right_allowed++;
+					info.right_spaces++;
+				} else {
+					info.left_spaces++;
+					info.left_req++;
+				}
 			}
 			break;
 
@@ -1705,31 +1804,22 @@ SUFFIX(go_format_number) (GString *result,
 				go_string_append_gstring (result, format_get_thousand ());
 			break;
 
-		/* FIXME: this is a gross hack */
-		/* FIXME: Missing support for scientific notation.
-		 * #00.00e###  that keeps axponent to a multiple of 3
-		 * XL seems to just special case that.
-		 **/
-		case 'E': case 'e': {
-			gboolean const is_lower = (*format++ == 'e');
-			gboolean shows_plus = FALSE;
-			int prec = info.right_optional + info.right_req;
-
-			can_render_number = TRUE;
-			if (*format == '+') {
-				shows_plus = TRUE;
-				format++;
-			} else if (*format == '-')
-				format++;
-
-			while (*format == '0' || *format == '#')
-				format++;
-
-			g_string_append_printf (result,
-						is_lower ? "%.*" FORMAT_e : "%.*" FORMAT_E,
-						prec, number);
-			return;
-		}
+		case 'E': 
+			if (info.exponent_seen) 
+				info.use_markup = TRUE;
+			else {
+				info.exponent_seen = TRUE;
+				can_render_number = TRUE;
+			}
+			break;
+			
+		case 'e': 
+			if (!info.exponent_seen) {
+				info.exponent_seen = TRUE;
+				info.exponent_lower_e = TRUE;
+				can_render_number = TRUE;
+			};
+			break;
 
 		case '\\':
 			if (format[1] != '\0') {
@@ -1797,7 +1887,7 @@ SUFFIX(go_format_number) (GString *result,
 						info.rendered = TRUE;
 						numerator += whole * denominator;
 					} else
-						show_zero = (number == 0);
+						show_zero = (number == 0 || whole == 0);
 
 					/*
 					 * FIXME: the space-aligning here doesn't come out
@@ -1818,9 +1908,13 @@ SUFFIX(go_format_number) (GString *result,
 				}
 			}
 
+		case '+':
+			if (info.exponent_seen) {
+				info.exponent_show_sign = TRUE;
+				break;
+			}
 		case '-':
 		case '(':
-		case '+':
 		case ':':
 		case ' ': /* eg # ?/? */
 		case '$':
@@ -1939,7 +2033,6 @@ SUFFIX(go_format_number) (GString *result,
 					/* HACK for fractional seconds.  */
 					DOUBLE days, secs;
 					int decs = 0;
-					int old_len = result->len;
 					format++;
 					while (format[1] == '0')
 						decs++, format++;
@@ -1947,10 +2040,12 @@ SUFFIX(go_format_number) (GString *result,
 					secs = SUFFIX(modf) (SUFFIX(fabs) (number), &days);
 					secs = SUFFIX(modf) (secs * (24 * 60 * 60), &days);
 
-					if (decs > 0)
+					if (decs > 0) {
+						size_t old_len = result->len;
 						g_string_append_printf (result, "%.*" FORMAT_f, decs, secs);
-					/* Remove the "0" or "1" before the dot.  */
-					g_string_erase (result, old_len, 1);
+						/* Remove the "0" or "1" before the dot.  */
+						g_string_erase (result, old_len, 1);
+					}
 				}
 			}
 			break;
@@ -2143,7 +2238,7 @@ go_fmt_general_int (GString *result, int val, int col_width)
 
 #ifdef DEFINE_COMMON
 void
-number_format_init (void)
+go_number_format_init (void)
 {
 	style_format_hash = g_hash_table_new_full (g_str_hash, g_str_equal,
 		NULL, (GDestroyNotify) go_format_unref);
@@ -2171,7 +2266,7 @@ cb_format_leak (gpointer key, gpointer value, gpointer user_data)
 
 #ifdef DEFINE_COMMON
 void
-number_format_shutdown (void)
+go_number_format_shutdown (void)
 {
 	GHashTable *tmp;
 
@@ -2580,9 +2675,13 @@ go_format_new (GOFormatFamily family, GOFormatDetails const *info)
 
 #ifdef DEFINE_COMMON
 /**
- * go_format_str_as_XL
+ * go_format_str_as_XL:
+ * @str: a format string
+ * @localized : should the string be in cannonical or locale specific form.
  *
  * The caller is responsible for freeing the resulting string.
+ *
+ * returns: a newly allocated string.
  */
 char *
 go_format_str_as_XL (char const *ptr, gboolean localized)
@@ -2662,11 +2761,11 @@ go_format_str_as_XL (char const *ptr, gboolean localized)
 
 #ifdef DEFINE_COMMON
 /**
- * go_format_as_XL :
- * @gf :
+ * go_format_as_XL:
+ * @fmt: a #GOFormat
  * @localized : should the string be in cannonical or locale specific form.
  *
- * Return a string which the caller is responsible for freeing.
+ * Returns: a string which the caller is responsible for freeing.
  */
 char *
 go_format_as_XL (GOFormat const *fmt, gboolean localized)
@@ -2693,9 +2792,10 @@ go_format_eq (GOFormat const *a, GOFormat const *b)
 #ifdef DEFINE_COMMON
 /**
  * go_format_ref :
- * @gf :
+ * @fmt : a #GOFormat
+ *
+ * Adds a reference to a GOFormat.
  *
- * Add a reference to a GOFormat
  **/
 GOFormat *
 go_format_ref (GOFormat *gf)
@@ -2715,9 +2815,10 @@ go_format_ref (GOFormat *gf)
 #ifdef DEFINE_COMMON
 /**
  * go_format_unref :
- * @gf :
+ * @fmt : a #GOFormat
+ *
+ * Removes a reference to @fmt, freeing when it goes to zero.
  *
- * Remove a reference to a GOFormat, freeing when it goes to zero.
  **/
 void
 go_format_unref (GOFormat *gf)
