@@ -26,7 +26,6 @@
 #include <pango/pango-attributes.h>
 #include <glib.h>
 
-#include <goffice/utils/goffice-utils.h>
 #include <glib-object.h>
 
 G_BEGIN_DECLS
@@ -59,6 +58,9 @@ typedef struct {
 	gboolean date_has_days;
 	gboolean date_has_months;
 	int      fraction_denominator;
+	gboolean simplify_mantissa;
+	int	 exponent_step;
+	gboolean use_markup;
 } GOFormatDetails;
 
 struct _GOFormat {
@@ -120,12 +122,20 @@ extern GOFormatCurrency     const go_format_currencies [];
 
 typedef struct {
 	int  right_optional, right_spaces, right_req, right_allowed;
-	int  left_spaces, left_req;
+	int  left_optional, left_spaces, left_req;
 	float scale;
 	gboolean rendered;
 	gboolean decimal_separator_seen;
 	gboolean group_thousands;
 	gboolean has_fraction;
+
+	gboolean exponent_seen;
+	
+	int exponent_digit_nbr;
+	gboolean exponent_show_sign;
+	gboolean exponent_lower_e;
+
+	gboolean use_markup;
 } GONumberFormat;
 
 void go_render_number (GString *result, double number, GONumberFormat const *info);
@@ -145,15 +155,10 @@ GString const *format_get_thousand     (void);
 GString const *format_get_decimal      (void);
 char const *   format_boolean          (gboolean b);
 
-void number_format_init (void);
-void number_format_shutdown (void);
-void currency_date_format_init     (void);
-void currency_date_format_shutdown (void);
-
-extern char const * const day_short   [];
-extern char const * const day_long    [];
-extern char const * const month_short [];
-extern char const * const month_long  [];
+void go_number_format_init (void);
+void go_number_format_shutdown (void);
+void go_currency_date_format_init     (void);
+void go_currency_date_format_shutdown (void);
 
 G_END_DECLS
 
