@@ -25,7 +25,7 @@
 #include "go-line.h"
 #include "go-math.h"
 
-#ifdef WITH_GTK
+#ifdef GOFFICE_WITH_GTK
 #include <goffice/gtk/go-combo-pixmaps.h>
 #endif
 
@@ -42,7 +42,7 @@ static const GOLineDashDesc line_dot_desc = 		{2,	{ 3, 3 } };
 static const GOLineDashDesc line_dash_dot_desc =	{4, 	{ 9, 3, 6, 3 } };
 static const GOLineDashDesc line_dash_dot_dot_desc =  {6, 	{ 9, 3, 3, 3, 3, 3 } };	
 
-struct {
+static struct {
 	GOLineDashType type;
 	char const *label;
 	char const *name;
@@ -239,7 +239,7 @@ go_line_dash_vpath (ArtVpath const *path,
 	return dashed;
 }
 					 
-#ifdef WITH_GTK
+#ifdef GOFFICE_WITH_GTK
 
 #define LINE_SAMPLE_HEIGHT	5
 #define LINE_SAMPLE_WIDTH	60
@@ -451,4 +451,4 @@ go_line_build_vpath (double const *x, double const *y, int n)
 	return path;
 }
 
-#endif /* WITH_GTK */
+#endif /* GOFFICE_WITH_GTK */
