@@ -24,7 +24,7 @@
 #include "go-color.h"
 #include <goffice/utils/go-math.h>
 
-#ifdef WITH_GTK
+#ifdef GOFFICE_WITH_GTK
 #include <goffice/gtk/go-combo-color.h>
 #include <goffice/gtk/go-combo-pixmaps.h>
 #include <gdk-pixbuf/gdk-pixdata.h>
@@ -45,6 +45,21 @@
 #define MARKER_DEFAULT_SIZE 5
 #define MARKER_OUTLINE_WIDTH 0.1
 
+struct _GOMarker {
+	GObject 	base;
+
+	int		size;
+	double		scale;
+	GOMarkerShape	shape;
+	GOColor		outline_color;
+	GOColor		fill_color;
+#ifdef GOFFICE_WITH_GTK
+	GdkPixbuf	*pixbuf;
+#else
+	gpointer	 pixbuf_placeholder;
+#endif
+};
+
 typedef struct {
 	GObjectClass	base;
 } GOMarkerClass;
@@ -241,7 +256,7 @@ static MarkerShape const marker_shapes[GO_MARKER_MAX] = {
 
 static GObjectClass *marker_parent_klass;
 
-#ifdef WITH_GTK
+#ifdef GOFFICE_WITH_GTK
 static GdkPixbuf *
 new_blank_pixbuf (GOMarker *marker, guint size)
 {
@@ -332,13 +347,11 @@ marker_free_pixbuf (GOMarker * marker)
 static void
 marker_update_pixbuf (GOMarker * marker)
 {
-	if (marker->pixbuf)
-		marker_free_pixbuf (marker);
 	marker->pixbuf = marker_create_pixbuf_with_size (marker, marker->size);
 }
 #else
 # define marker_free_pixbuf(marker) do {} while(0)
-#endif /* WITH_GTK */
+#endif /* GOFFICE_WITH_GTK */
 
 static void
 go_marker_finalize (GObject *obj)
@@ -355,7 +368,7 @@ go_marker_init (GOMarker * marker)
 	marker->outline_color	= RGBA_BLACK;
 	marker->fill_color	= RGBA_WHITE;
 	marker->size		= MARKER_DEFAULT_SIZE;
-#ifdef WITH_GTK
+#ifdef GOFFICE_WITH_GTK
 	marker->pixbuf 		= NULL;
 #endif
 	marker->scale		= 1.;
@@ -394,7 +407,7 @@ go_marker_get_paths (GOMarker * marker,
 	*fill_path = marker_shapes[marker->shape].fill_path;
 }
 
-#ifdef WITH_GTK
+#ifdef GOFFICE_WITH_GTK
 GdkPixbuf *
 go_marker_get_pixbuf (GOMarker * marker, double scale)
 {
@@ -415,7 +428,7 @@ go_marker_get_pixbuf_with_size (GOMarker *marker, guint size)
 
 	return marker_create_pixbuf_with_size (marker, size);
 }
-#endif /* WITH_GTK */
+#endif /* GOFFICE_WITH_GTK */
 
 GOMarkerShape 
 go_marker_get_shape (GOMarker * marker)
@@ -502,9 +515,9 @@ go_marker_assign (GOMarker *dst, GOMarker const *src)
 	dst->outline_color	= src->outline_color;
 	dst->fill_color		= src->fill_color;
 
-#ifdef WITH_GTK
+#ifdef GOFFICE_WITH_GTK
 	if (dst->pixbuf != NULL)
-		g_object_unref (dst->pixbuf);
+		g_object_unref (src->pixbuf);
 	dst->pixbuf = src->pixbuf;
 	if (dst->pixbuf != NULL)
 		g_object_ref (dst->pixbuf);
@@ -531,7 +544,7 @@ GSF_CLASS (GOMarker, go_marker,
 
 /*---------------------------------------------------------------------------*/
 
-#ifdef WITH_GTK
+#ifdef GOFFICE_WITH_GTK
 gpointer
 go_marker_selector (GOColor outline_color, GOColor fill_color,
 		    GOMarkerShape default_shape)
@@ -580,4 +593,4 @@ go_marker_selector (GOColor outline_color, GOColor fill_color,
 
 	return GTK_WIDGET (w);
 }
-#endif /* WITH_GTK */
+#endif /* GOFFICE_WITH_GTK */
