@@ -25,7 +25,7 @@
 #include "go-pattern.h"
 #include "go-color.h"
 
-#ifdef WITH_GTK
+#ifdef GOFFICE_WITH_GTK
 #include <goffice/gtk/go-combo-pixmaps.h>
 #include <gdk-pixbuf/gdk-pixdata.h>
 #endif
@@ -120,10 +120,11 @@ go_pattern_is_solid (GOPattern const *pat, GOColor *color)
 
 /**
  * go_pattern_set_solid :
- * @pat  : #GOPattern
- * @fore : #GOColor
+ * @pat  : a #GOPattern
+ * @fore : a #GOColor
  *
  * Makes @pat a solid pattern with colour @fore.
+ *
  **/
 void
 go_pattern_set_solid (GOPattern *pat, GOColor fore)
@@ -142,15 +143,17 @@ go_pattern_get_pattern (GOPattern const *pat)
 
 /**
  * go_pattern_get_svg_path:
- * @pattern: #GOPattern
- * @double:  pattern width
+ * @pattern: a #GOPattern
+ * @width:  pattern width
  * @height:  pattern height
  *
- * Returns an SVG path as string, which represents pattern shape.
+ * Retrieves an SVG path as string, which represents pattern shape.
  * Caller is responsible for freeing the resulting string.
  *
  * If width != NULL, returns pattern width.
  * If height != NULL, returns pattern height.
+ *
+ * returns: a #char buffer.
  **/
 #warning This result is actually an xmlChar we could run into trouble with g_free vs xmlFree.  Can we change the interface ?
 char *
@@ -199,7 +202,7 @@ go_pattern_get_svg_path (GOPattern const *pattern, double *width, double *height
 	return (char *)svg_path;
 }
 
-#ifdef WITH_GTK
+#ifdef GOFFICE_WITH_GTK
 gpointer
 go_pattern_selector (GOColor fore, GOColor back,
 		     GOPatternType default_pat)
@@ -262,7 +265,7 @@ go_pattern_selector (GOColor fore, GOColor back,
 	art_svp_free (svp);
 	return w;
 }
-#endif /* WITH_GTK */
+#endif /* GOFFICE_WITH_GTK */
 
 /*
  *  A slightly modified version of art_rgb_svp to render into rgba buffer
