@@ -467,8 +467,8 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 						paths[j][i * 2 - 1].x = gog_axis_map_to_view (x_map, x);
 						paths[j][i * 2 - 1].y = gog_axis_map_to_view (y_map, work.y);
 					}
-					paths[j][i * 2].x = gog_axis_map_to_view (x_map, x);
-					paths[j][i * 2].y = gog_axis_map_to_view (y_map, work.y + work.h);
+					paths[j][i].x = gog_axis_map_to_view (x_map, x);
+					paths[j][i].x = gog_axis_map_to_view (y_map, work.y + work.h);
 				}
 			}
 		}
@@ -495,23 +495,10 @@ gog_barcol_view_render (GogView *view, GogViewAllocation const *bbox)
 	gog_chart_map_free (chart_map);
 }
 
-static gboolean
-gog_barcol_view_info_at_point (GogView *view, double x, double y,
-			       GogObject const *cur_selection,
-			       GogObject **obj, char **name)
-{
-	if (obj != NULL)
-		*obj = view->model;
-	if (name != NULL)
-		*name = g_strdup (gog_object_get_name (GOG_OBJECT (view->model)));
-	return TRUE;
-}
-
 static void
 gog_barcol_view_class_init (GogViewClass *view_klass)
 {
 	view_klass->render	  = gog_barcol_view_render;
-	view_klass->info_at_point = gog_barcol_view_info_at_point;
 	view_klass->clip	  = TRUE;
 }
 
