@@ -226,7 +226,6 @@ gog_dropbar_view_render (GogView *view, GogViewAllocation const *bbox)
 		} else
 			path1[j] = NULL;
 		for (i = 0; i < n; i++) {
-			x++;
 			work.x = x;
 			work.y = start_vals[i];
 			work.h = end_vals[i] - work.y;
@@ -248,6 +247,7 @@ gog_dropbar_view_render (GogView *view, GogViewAllocation const *bbox)
 					barcol_draw_rect (view->renderer, model->horizontal, x_map, y_map, &work);
 			barcol_draw_rect (view->renderer, model->horizontal, x_map, y_map, &work);
 			gog_renderer_pop_style (view->renderer);
+			x += 1;
 		}
 		offset += step;
 		g_object_unref (neg_style);
@@ -268,23 +268,10 @@ gog_dropbar_view_render (GogView *view, GogViewAllocation const *bbox)
 	gog_axis_map_free (y_map);
 }
 
-static gboolean
-gog_dropbar_view_info_at_point (GogView *view, double x, double y,
-			       GogObject const *cur_selection,
-			       GogObject **obj, char **name)
-{
-	if (obj != NULL)
-		*obj = view->model;
-	if (name != NULL)
-		*name = g_strdup (gog_object_get_name (GOG_OBJECT (view->model)));
-	return TRUE;
-}
-
 static void
 gog_dropbar_view_class_init (GogViewClass *view_klass)
 {
 	view_klass->render	  = gog_dropbar_view_render;
-	view_klass->info_at_point = gog_dropbar_view_info_at_point;
 	view_klass->clip	  = TRUE;
 }
 
